/*************************************************************************
*
* YahooPOPs!
*
* YahooPOPs! is an application that provides POP3 access to Yahoo! Mail.
* You do not need to enable POP3 access with Yahoo! to use this program.
* This program works within the license agreement of Yahoo! to provide
* you with POP3 access.
*
* YahooPOPs! emulates a POP3 mail server and should work with most of the
* email clients. Since YahooPOPs! is not a full-fledged POP3 mail server
* you might find it not working with all mail clients. Do keep us informed
* about such mail clients so that we can enhance YahooPOPs!
*
* Official Website: http://yahoopops.sourceforge.net
*
*-------------------------------------------------------------------------
*
* Name    :	ConfigMiscPage.cpp
*
* Purpose : Configuration Dialog - Misc Page
*
* Authors : Anuj Seth
* 			K.Shyam
*
* License : GNU Public License (Version 2 or later)
*
*-------------------------------------------------------------------------
*
* THIS SOFTWARE IS PROVIDED ``AS IS'' BY THE AUTHORS AND INDEPENDENT
* CONTRIBUTORS AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
* LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
* PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR 
* CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
* EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
* PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
* PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
* LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
* NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
* SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
*************************************************************************/
#include "stdafx.h"

#ifdef WIN32
#include "YahooPOPs.h"
#include "ConfigMiscPage.h"
#include <commctrl.h>
#include <shlobj.h>

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

extern CDataHolder settings;

IMPLEMENT_DYNCREATE(CConfigMiscPage, CPropertyPage)

CConfigMiscPage::CConfigMiscPage() : CCustomPropertyPage(CConfigMiscPage::IDD)
{
	//{{AFX_DATA_INIT(CConfigMiscPage)
	m_nUserAgent = 0;
	m_bQuietMode = FALSE;
	m_bAutoStartWithWindows = FALSE;
	m_bAlwaysOnTop = FALSE;
	m_bHideTrayIcon = FALSE;
	m_bEnableUIDL = FALSE;
	m_bIgnoreSize = FALSE;
	//}}AFX_DATA_INIT
}

CConfigMiscPage::~CConfigMiscPage()
{
}

void CConfigMiscPage::DoDataExchange(CDataExchange* pDX)
{
	CPropertyPage::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CConfigMiscPage)
	DDX_Control(pDX, IDC_USER_AGENT, m_ctlUserAgent);
	DDX_CBIndex(pDX, IDC_USER_AGENT, m_nUserAgent);
	DDX_Check(pDX, IDC_QUIET_MODE, m_bQuietMode);
	DDX_Check(pDX, IDC_AUTO_START, m_bAutoStartWithWindows);
	DDX_Check(pDX, IDC_ALWAYS_ON_TOP, m_bAlwaysOnTop);
	DDX_Check(pDX, IDC_HIDE_ICON, m_bHideTrayIcon);
	DDX_Check(pDX, IDC_ENABLE_UIDL, m_bEnableUIDL);
	DDX_Check(pDX, IDC_IGNORE_SIZE, m_bIgnoreSize);
	//}}AFX_DATA_MAP
}

/////////////////////////////////////////////////////////////////////////////////////////////////
//		GetShellFolderPath - Function to Get the Path to the Shell Folders 					   //
//																							   //
//		Return Type : bool																	   //
//					: Returns true if successfull or false if errored.						   //
//																							   //
//		Parameters  : LPCTSTR lpShellFolder													   //
//					: should contain one of the following,									   //
//						AppData, Cache, Cookies, Desktop, Favorites, Fonts, History, NetHood,  //
//						Personal, Printhood, Programs, Recent, SendTo, Start Menu, Startup,	   //
//						Templates, ShellNew													   //
//																							   //
//					: LPSTR lpShellPath														   //
//						will be populated with full path of the desired Shell Folder.		   //
/////////////////////////////////////////////////////////////////////////////////////////////////
bool GetShellFolderPath(LPCSTR lpShellFolder, LPSTR lpShellPath)
{
	//lpShellFolder can be 
	//AppData,Cache,Cookies,Desktop,Favorites,Fonts,History,NetHood,
	//Personal,Printhood,Programs,Recent,SendTo,Start Menu,Startup,
	//Templates, ShellNew
	DWORD rc;
	DWORD length = MAX_PATH;
	DWORD type = REG_SZ;
    HKEY hkey;
	
	rc = RegOpenKeyEx(HKEY_CURRENT_USER,
			"Software\\Microsoft\\Windows\\CurrentVersion\\Explorer\\Shell Folders",
			0, KEY_READ, &hkey);
	
	if (rc == ERROR_SUCCESS)
	{
		rc = RegQueryValueEx(hkey, lpShellFolder, NULL, &type,
			(BYTE *) lpShellPath, &length);
		RegCloseKey(hkey);
	}
	
	if (rc == ERROR_SUCCESS)
		return true;
	else
		return false;
}

/////////////////////////////////////////////////////////////////////////////////////////////////
//		CreateShortCut - Function to Create Short Cut					 					   //
//																							   //
//		Return Type : bool																	   //
//					: Returns true if successfull or false if errored.						   //
//																							   //
//		Parameters  : LPCTSTR lpszSourceFile												   //
//					: should contain the Source file for which the shourtcut has to be created.//
//																							   //
//					: LPCSTR lpszDestination												   //
//						Should contain the target directory and the shourt cut file name.	   //
//																							   //
//					: LPCSTR lpszDesc														   //
//						can contain any suitable description.								   //
/////////////////////////////////////////////////////////////////////////////////////////////////
bool CreateShortCut(LPCSTR lpszSourceFile,
					LPCSTR lpszDestination, 
					LPCSTR lpszDesc,
					LPCSTR lpszDirectory) 
{ 
    HRESULT		hResult    = NULL; 
    IShellLink* pShellLink = NULL; 
	bool		rc		   = false;
	
	hResult = CoInitialize(NULL);
    // Initialize the IShellLink interface. 
    
	hResult = CoCreateInstance(CLSID_ShellLink, NULL, 
        CLSCTX_INPROC_SERVER, IID_IShellLink, (void**) &pShellLink); 
	// Get a pointer to the IShellLink Interface
	
    if (SUCCEEDED(hResult)) 
	{ 
        IPersistFile* ppf = NULL; 
		
        pShellLink->SetPath(lpszSourceFile); 
        // Set the path to the shortcut target

		pShellLink->SetWorkingDirectory(lpszDirectory);
        
		hResult = pShellLink->SetDescription(lpszDesc); 
		// Add Description
		
        hResult = pShellLink->QueryInterface(IID_IPersistFile, (void**) &ppf); 
		// Query IShellLink for the IPersistFile interface for saving the 
        // shortcut in persistent storage. 
		
        if (SUCCEEDED(hResult)) 
		{ 
            WORD wszWideString[MAX_PATH]; 
			
            // Ensure that the string is ANSI. 
            MultiByteToWideChar(CP_ACP, 0, lpszDestination, -1,
				wszWideString, MAX_PATH); 
			
            hResult = ppf->Save(wszWideString, TRUE); 
            // Save the link by calling IPersistFile::Save. 
            
			ppf->Release(); 
			rc = true;
        } 
        pShellLink->Release(); 
		CoUninitialize();
		// Clean up
    } 
	return rc; 
} 

/////////////////////////////////////////////////////////////////////////////////////////////////
//		LaunchAtStartup - Function to Launch a File or Program during Startup				   //
//					(by creating a short cut in the startup folder)							   //
//																							   ////		Return Type : bool																	   //
//					: Returns true if successfull or false if errored.						   //
//																							   //
//		Parameters  : LPCTSTR lpszFileName													   //
//					: should contain the Full File Name for your Application.				   //
/////////////////////////////////////////////////////////////////////////////////////////////////
bool LaunchAtStartup(LPCTSTR lpszFileName, 
					 LPCTSTR lpszDescription,
					 LPCTSTR lpszDirectory)
{
	char szDestFileName[MAX_PATH] = "";
	char cBackwardSlash			= '\\';
	
	if (lpszFileName == NULL)
		return false;
	// File Name cannot be empty
	
	if(! GetShellFolderPath("Startup",szDestFileName))
		return false;
	// Error getting Startup Shell Folder Path
	
	strcat(szDestFileName,strrchr(lpszFileName,cBackwardSlash));
	strcat(szDestFileName,".lnk");
	
	return CreateShortCut(lpszFileName, szDestFileName, lpszDescription, lpszDirectory);
	// Create a Short up for the file or application in the start up so that 
	// the file or application starts up during Windows Startup.
}

BEGIN_MESSAGE_MAP(CConfigMiscPage, CCustomPropertyPage)
	//{{AFX_MSG_MAP(CConfigMiscPage)
		// NOTE: the ClassWizard will add message map macros here
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

void CConfigMiscPage::Init(CDataHolder &data)
{
	m_nUserAgent = data.m_nUserAgent;
	m_bQuietMode = data.m_bQuietMode;
	m_bAutoStartWithWindows = data.m_bAutoStartWithWindows;
    m_bAlwaysOnTop = data.m_bAlwaysOnTop;
    m_bEnableUIDL = data.m_bEnableUIDL;
    m_bHideTrayIcon = data.m_bHideTrayIcon;
	m_bIgnoreSize = data.m_bIgnoreSize;

	UpdateData(FALSE);
}

BOOL CConfigMiscPage::ValidateData(void)
{
	char shortcut[MAX_PATH] = "";

	UpdateData(TRUE);

	if(m_bAutoStartWithWindows)
	{
		/* Create shortcut */
		char yahoopopsPath[MAX_PATH];	
		char yahoopops[MAX_PATH];

		GetCurrentDirectory(sizeof(yahoopopsPath), yahoopopsPath);
		GetCurrentDirectory(sizeof(yahoopopsPath), yahoopops);
		strcat(yahoopops, "\\YahooPOPs");
		LaunchAtStartup(yahoopops, 
						"Launch YahooPOPs! at Windows startup",
						yahoopopsPath);
	}
	else
	{
		/* Remove shortcut */
		if(GetShellFolderPath("Startup", shortcut))
		{
			strcat(shortcut, "\\YahooPOPs.lnk");

            try
            {
			    unlink(shortcut);
            }
            catch(...)
            {
                /* Do nothing */
            }
		}
	}

	return TRUE;
}

void CConfigMiscPage::SaveData(CDataHolder &data)
{
	CString str;
	m_ctlUserAgent.GetLBText(m_nUserAgent, str);

	settings.m_nUserAgent = m_nUserAgent;
	settings.m_strUserAgent = str;
	settings.m_bQuietMode = m_bQuietMode;
	settings.m_bAutoStartWithWindows = m_bAutoStartWithWindows;
    settings.m_bAlwaysOnTop = m_bAlwaysOnTop;
    settings.m_bEnableUIDL = m_bEnableUIDL;
	settings.m_bHideTrayIcon = m_bHideTrayIcon;
	settings.m_bIgnoreSize = m_bIgnoreSize;

	data.m_nUserAgent = m_nUserAgent;
	data.m_strUserAgent = str;
	data.m_bQuietMode = m_bQuietMode;
	data.m_bAutoStartWithWindows = m_bAutoStartWithWindows;
    data.m_bAlwaysOnTop = m_bAlwaysOnTop;
    data.m_bEnableUIDL = m_bEnableUIDL;
	data.m_bHideTrayIcon = m_bHideTrayIcon;
	data.m_bIgnoreSize = m_bIgnoreSize;
}

#endif // #ifdef WIN32
