/*************************************************************************
*
* YahooPOPs!
*
* YahooPOPs! is an application that provides POP3 access to Yahoo! Mail.
* You do not need to enable POP3 access with Yahoo! to use this program.
* This program works within the license agreement of Yahoo! to provide
* you with POP3 access.
*
* YahooPOPs! emulates a POP3 mail server and should work with most of the
* email clients. Since YahooPOPs! is not a full-fledged POP3 mail server
* you might find it not working with all mail clients. Do keep us informed
* about such mail clients so that we can enhance YahooPOPs!
*
* Official Website: http://yahoopops.sourceforge.net
*
*-------------------------------------------------------------------------
*
* Name    :	ConfigNetworkPage.cpp
*
* Purpose : Configuration Dialog - Network Settings Page
*
* Authors : Anuj Seth
* 			K.Shyam
*
* License : GNU Public License (Version 2 or later)
*
*-------------------------------------------------------------------------
*
* THIS SOFTWARE IS PROVIDED ``AS IS'' BY THE AUTHORS AND INDEPENDENT
* CONTRIBUTORS AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
* LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
* PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR 
* CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
* EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
* PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
* PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
* LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
* NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
* SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
*************************************************************************/
#include "stdafx.h"
#ifdef WIN32
#include "YahooPOPs.h"
#include "ConfigNetworkPage.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

extern CDataHolder settings;

IMPLEMENT_DYNCREATE(CConfigNetworkPage, CPropertyPage)

CConfigNetworkPage::CConfigNetworkPage() : CCustomPropertyPage(CConfigNetworkPage::IDD)
{
	//{{AFX_DATA_INIT(CConfigNetworkPage)
	m_nPort = 110;
	m_nIPAddr = 0;
	m_nSmtpPort = 25;
	m_bEnableSMTP = FALSE;
	//}}AFX_DATA_INIT
}

CConfigNetworkPage::~CConfigNetworkPage()
{
}

void CConfigNetworkPage::DoDataExchange(CDataExchange* pDX)
{
	CPropertyPage::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CConfigNetworkPage)
	DDX_Control(pDX, IDC_SMTP_PORT, m_ctlSmtpPort);
	DDX_Control(pDX, IDC_IP_ADDR, m_ctlIPAddr);
	DDX_Text(pDX, IDC_PORT, m_nPort);
	DDV_MinMaxUInt(pDX, m_nPort, 1, 65535);
	DDX_CBIndex(pDX, IDC_IP_ADDR, m_nIPAddr);
	DDX_Text(pDX, IDC_SMTP_PORT, m_nSmtpPort);
	DDV_MinMaxUInt(pDX, m_nSmtpPort, 1, 65535);
	DDX_Check(pDX, IDC_ENABLE_SMTP, m_bEnableSMTP);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CConfigNetworkPage, CPropertyPage)
	//{{AFX_MSG_MAP(CConfigNetworkPage)
	ON_BN_CLICKED(IDC_ENABLE_SMTP, OnEnableSmtp)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

void CConfigNetworkPage::Init(CDataHolder &data)
{
	m_nPort = data.m_nNetworkPort;
	m_nSmtpPort = data.m_nSmtpPort;
    m_bEnableSMTP = data.m_bEnableSmtp;

    m_ctlSmtpPort.EnableWindow(m_bEnableSMTP);

	// Get the list of IP addresses for this machine
	GetIPAddressList(data);

	UpdateData(FALSE);
}

BOOL CConfigNetworkPage::ValidateData(void)
{
	UpdateData(TRUE);

	if(m_nPort == m_nSmtpPort)
	{
		AfxMessageBox("POP3 port must be different from the SMTP port.", MB_OK | MB_ICONEXCLAMATION);
		return FALSE;
	}

	if(m_nPort != settings.m_nNetworkPort || m_nSmtpPort != settings.m_nSmtpPort || m_bEnableSMTP != settings.m_bEnableSmtp)
	{
		AfxMessageBox("Any changes to the network settings will take affect when you restart YahooPOPs!", MB_OK | MB_ICONEXCLAMATION);
	}

	return TRUE;
}

void CConfigNetworkPage::SaveData(CDataHolder &data)
{
	CString str;
	m_ctlIPAddr.GetLBText(m_nIPAddr, str);

	settings.m_nNetworkPort = m_nPort;
	settings.m_strNetworkIPAddr = str;
	settings.m_nSmtpPort = m_nSmtpPort;
    settings.m_bEnableSmtp = m_bEnableSMTP;

	data.m_nNetworkPort = m_nPort;
	data.m_strNetworkIPAddr = str;
	data.m_nSmtpPort = m_nSmtpPort;
    data.m_bEnableSmtp = m_bEnableSMTP;
}

void CConfigNetworkPage::GetIPAddressList(CDataHolder &data)
{
	int rc=0;
	char buf[256];
	struct hostent *myaddr=0;
	struct in_addr in_ip;
	CString ip;
	bool localhost = false;

	//get the host address
	if(gethostname(buf, 256) == SOCKET_ERROR)
	{
		ASSERT(FALSE);
	}
	else if((myaddr = gethostbyname(buf)) == (hostent *)NULL)
	{
		ASSERT(FALSE);
	}
	else if(myaddr->h_length != 4)
	{
		ASSERT(FALSE);
	}
	else
	{
		for(int i(0); myaddr->h_addr_list[i] != 0; i++)
		{
			//copy the ip address into in_addr structure
			in_ip.S_un.S_un_b.s_b1 = myaddr->h_addr_list[i][0];
			in_ip.S_un.S_un_b.s_b2 = myaddr->h_addr_list[i][1];
			in_ip.S_un.S_un_b.s_b3 = myaddr->h_addr_list[i][2];
			in_ip.S_un.S_un_b.s_b4 = myaddr->h_addr_list[i][3];

			ip = inet_ntoa(in_ip);
			m_ctlIPAddr.AddString(ip);

			if(ip == data.m_strNetworkIPAddr)
			{
				m_nIPAddr = rc;
			}
			
			if(ip == "127.0.0.1")
			{
				localhost = true;
			}

			rc++;
		}

		if(!localhost)
		{
			/* Add localhost */
			m_ctlIPAddr.AddString("127.0.0.1");

			if(data.m_strNetworkIPAddr == "127.0.0.1")
			{
				m_nIPAddr = rc;
			}
		}
	}
}

void CConfigNetworkPage::OnEnableSmtp() 
{
    UpdateData(TRUE);
    m_ctlSmtpPort.EnableWindow(m_bEnableSMTP);
}

#endif // #ifdef WIN32
