/*************************************************************************
*
* YahooPOPs!
*
* YahooPOPs! is an application that provides POP3 access to Yahoo! Mail.
* You do not need to enable POP3 access with Yahoo! to use this program.
* This program works within the license agreement of Yahoo! to provide
* you with POP3 access.
*
* YahooPOPs! emulates a POP3 mail server and should work with most of the
* email clients. Since YahooPOPs! is not a full-fledged POP3 mail server
* you might find it not working with all mail clients. Do keep us informed
* about such mail clients so that we can enhance YahooPOPs!
*
* Official Website: http://yahoopops.sourceforge.net
*
*-------------------------------------------------------------------------
*
* Name    :	HttpClient.cpp
*
* Purpose : A C++ class that provides with the basic HTTP GET and POST
*           methods. Proxy support is provided.
*
* Authors : Anuj Seth
* 			K.Shyam
*
* License : GNU Public License (Version 2 or later)
*
*-------------------------------------------------------------------------
*
* THIS SOFTWARE IS PROVIDED ``AS IS'' BY THE AUTHORS AND INDEPENDENT
* CONTRIBUTORS AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
* LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
* PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR 
* CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
* EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
* PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
* PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
* LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
* NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
* SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
*************************************************************************/
#include "stdafx.h"
#include <iostream>
#include <string.h>
#include <malloc.h>
#include "DataHolder.h"
#include "HttpClient.h"
#include "LogFile.h"
#include <curl/curl.h>
#include <curl/types.h>

typedef struct MemoryStruct_t
{
	char *memory;
	size_t size;
	HttpSession *session;
} MemoryStruct;

extern CDataHolder settings;
extern LogFile logFile;

static size_t WriteMemoryCallback(void *ptr, size_t size, size_t nmemb, void *data)
{
	register int realsize = size * nmemb;
	MemoryStruct *mem = (MemoryStruct *)data;
	
	mem->memory = static_cast<char *>(realloc(mem->memory, mem->size + realsize + 1));

	if (mem->memory) 
	{
		memcpy(&(mem->memory[mem->size]), ptr, realsize);
		mem->size += realsize;
		mem->memory[mem->size] = 0;
	}

	return realsize;
}

static size_t HttpHeaderData(void *ptr, size_t size, size_t nmemb, void *stream)
{
	register int realsize = size * nmemb;
	MemoryStruct *mem = (MemoryStruct *)stream;

	if(strnicmp((const char *)(ptr), "set-cookie", 10) == 0)
	{
		mem->session->AddCookie(static_cast<char *>(ptr));
	}

	return realsize;
}

/*******************************************************************
* Function  : HttpClient constructor
* Parameters: -
* Returns   : -
* Purpose   : Initialize the object
*******************************************************************/
HttpClient::HttpClient(HttpSession *s)
{
	session			= s;
	bUseHttpProxy	= FALSE;
	bUseHttpsProxy  = FALSE;
	pResponsePage	= NULL;
	responsePageLen = 0;
	host[0]			= '\0';
	headers			= NULL;

	/* init the curl session */
	curl_handle = curl_easy_init();
	
	curl_easy_setopt(curl_handle, CURLOPT_WRITEFUNCTION, WriteMemoryCallback);
	curl_easy_setopt(curl_handle, CURLOPT_HEADERFUNCTION, HttpHeaderData);
	curl_easy_setopt(curl_handle, CURLOPT_USERAGENT, LPCTSTR(settings.m_strUserAgent));
	curl_easy_setopt(curl_handle, CURLOPT_FOLLOWLOCATION, 1);
	curl_easy_setopt(curl_handle, CURLOPT_MAXREDIRS, 10);

	/* set the headers (including cookies) for the session */
	curl_easy_setopt(curl_handle, CURLOPT_COOKIEFILE, "cookies.tmp");

	/* set the SSL/TLS defaults */
	curl_easy_setopt(curl_handle, CURLOPT_SSLENGINE_DEFAULT, 1);
	curl_easy_setopt(curl_handle, CURLOPT_SSL_VERIFYPEER, 0);
    curl_easy_setopt(curl_handle, CURLOPT_SSL_VERIFYHOST, 0);
	curl_easy_setopt(curl_handle, CURLOPT_NOPROGRESS, 1);
	curl_easy_setopt(curl_handle, CURLOPT_NOSIGNAL, 1);
}

/*******************************************************************
* Function  : HttpClient desctructor
* Parameters: -
* Returns   : -
* Purpose   : Object cleanup
*******************************************************************/
HttpClient::~HttpClient()
{
	if(pResponsePage != NULL)
	{
		free(pResponsePage);
	}

	/* cleanup curl stuff */
	curl_easy_cleanup(curl_handle);

    if(headers != NULL)
    {
   	    curl_slist_free_all(headers);
    }
}

/*******************************************************************
* Function  : SetHttpProxy()
* Parameters: proxyHost, proxyPort - HTTP Proxy
* Returns   : -
* Purpose   : To set the HTTP proxy values
*******************************************************************/
void HttpClient::SetHttpProxy(const char *proxyHost, unsigned int proxyPort)
{
	strcpy(httpProxyHost, proxyHost);
	httpProxyPort = proxyPort;
}

/*******************************************************************
* Function  : UseHttpProxy()
* Parameters: bUseProxy - Is the proxy to be used?
* Returns   : -
* Purpose   : To inform HttpClient whether the HTTP proxy is to be
*             used for the HTTP GET/POST requests
*******************************************************************/
void HttpClient::UseHttpProxy(BOOL bUseProxy)
{
	bUseHttpProxy = bUseProxy;
}

/*******************************************************************
* Function  : UseHttpsProxy()
* Parameters: bUseProxy - Is the proxy to be used?
* Returns   : -
* Purpose   : To inform HttpClient whether the HTTPS proxy is to be
*             used for the HTTPS GET/POST requests
*******************************************************************/
void HttpClient::UseHttpsProxy(BOOL bUseProxy)
{
	bUseHttpsProxy = bUseProxy;
}

/*******************************************************************
* Function  : SetHttpsProxy()
* Parameters: proxyHost, proxyPort - HTTPS Proxy
* Returns   : -
* Purpose   : To set the HTTPS proxy values
*******************************************************************/
void HttpClient::SetHttpsProxy(const char *proxyHost, unsigned int proxyPort)
{
	strcpy(httpsProxyHost, proxyHost);
	httpsProxyPort = proxyPort;
}

/*******************************************************************
* Function  : AddHeader()
* Parameters: name & value - The HTTP header to add to the request
* Returns   : TRUE - Header added
*             FALSE - Header not added
* Purpose   : To add a header for the HTTP GET/POST request
*******************************************************************/
BOOL HttpClient::AddHeader(const char *name, const char *value)
{
	char header[4096];

	if(name == NULL || value == NULL)
	{
		sprintf(header, "\r\n");
	}
	else
	{
		sprintf(header, "%s: %s", name, value);
	}

	headers = curl_slist_append(headers, header);

	return TRUE;
}

/*******************************************************************
* Function  : DoPost()
* Parameters: url - The URL to which the HTTP POST is to be done
*             postData - Pointer to the data to be posted
*             resp - The response object which will store the HTTP
*                    response received from the website
*             formflag - 'true' indicates a request to post data
*                    in multipart/form-data format. Default is false.
* Returns   : TRUE - Post succeeded (could have any HTTP errors,
*                    including 404 errors)
*             FALSE - The post failed most likely because a 
*                    connection coul not be established
* Purpose   : To do an HTTP Post. This is used during the Yahoo
*             login sequence and is retried a maximum of
*             HTTP_MAX_RETRIES times.
*******************************************************************/
BOOL HttpClient::DoPost(const char *url, const void * postData, HttpClientResponse &resp, bool formflag)
{
    BOOL retVal = 0;

    for(int i = 0; i < HTTP_MAX_RETRIES; i++)
    {
        if( (retVal = DoPostInternal(url, postData, resp, formflag)) == 0)
        {
            break;
        }
    }

    return retVal;
}

/*******************************************************************
* Function  : DoPostInternal()
* Parameters: url - The URL to which the HTTP POST is to be done
*             postData - Pointer to the data to be posted
*             resp - The response object which will store the HTTP
*                    response received from the website
*             formflag - 'true' indicates a request to post data
*                    in multipart/form-data format. Default is false.
* Returns   : TRUE - Post succeeded (could have any HTTP errors,
*                    including 404 errors)
*             FALSE - The post failed most likely because a 
*                    connection coul not be established
* Purpose   : To do an HTTP Post. This is used during the Yahoo
*             login sequence and to send messages
*******************************************************************/
BOOL HttpClient::DoPostInternal(const char *url, const void * postData, HttpClientResponse &resp, bool formflag)
{
	CURL *curl_handle;
	CURLcode res;
	MemoryStruct chunk;
	char strProxy[255];
	char strProxyAuth[255];
	
	chunk.memory = NULL; /* we expect realloc(NULL, size) to work */
	chunk.size   = 0;    /* no data at this point */
	chunk.session = session;

	/* init the curl session */
	curl_handle = curl_easy_init();

	session->GetCookies(this);
	
	/* specify URL to get */
	curl_easy_setopt(curl_handle, CURLOPT_URL, url);
	
	/* send all data to this function  */
	curl_easy_setopt(curl_handle, CURLOPT_WRITEFUNCTION, WriteMemoryCallback);
	curl_easy_setopt(curl_handle, CURLOPT_FILE, static_cast<void *>(&chunk));
	curl_easy_setopt(curl_handle, CURLOPT_HEADERFUNCTION, HttpHeaderData);
	curl_easy_setopt(curl_handle, CURLOPT_WRITEHEADER, static_cast<void *>(&chunk));

	curl_easy_setopt(curl_handle, CURLOPT_USERAGENT, LPCTSTR(settings.m_strUserAgent));
	curl_easy_setopt(curl_handle, CURLOPT_HTTPHEADER, headers);
	curl_easy_setopt(curl_handle, CURLOPT_FOLLOWLOCATION, 1);
	curl_easy_setopt(curl_handle, CURLOPT_MAXREDIRS, 10);

	/* set the headers (including cookies) for the session */
	curl_easy_setopt(curl_handle, CURLOPT_USERAGENT, LPCTSTR(settings.m_strUserAgent));
//	curl_easy_setopt(curl_handle, CURLOPT_COOKIEFILE, "cookies.tmp");

	/* set the SSL/TLS defaults */
	curl_easy_setopt(curl_handle, CURLOPT_SSLENGINE_DEFAULT, 1);
	curl_easy_setopt(curl_handle, CURLOPT_SSL_VERIFYPEER, 0);
    curl_easy_setopt(curl_handle, CURLOPT_SSL_VERIFYHOST, 0);
	curl_easy_setopt(curl_handle, CURLOPT_NOPROGRESS, 1);
	curl_easy_setopt(curl_handle, CURLOPT_NOSIGNAL, 1);

	/* Set the operation to POST */
	if (formflag)
    {
	    curl_easy_setopt(curl_handle, CURLOPT_HTTPPOST, postData);
    }
	else
    {
		curl_easy_setopt(curl_handle, CURLOPT_POSTFIELDS, postData);
    }
	
	if(strnicmp(url, "https://", 8) == 0)
	{
		if(bUseHttpsProxy)
		{
			sprintf(strProxy, "%s:%d", httpsProxyHost, httpsProxyPort);
			curl_easy_setopt(curl_handle, CURLOPT_PROXY, strProxy);
		}
	}
	else if(bUseHttpProxy)
	{
		sprintf(strProxy, "%s:%d", httpProxyHost, httpProxyPort);
		curl_easy_setopt(curl_handle, CURLOPT_PROXY, strProxy);
	}

	if(settings.m_bProxyAuth == TRUE)
	{
		sprintf(strProxyAuth, "%s:%s", LPCTSTR(settings.m_strProxyAuthUsername), LPCTSTR(settings.m_strProxyAuthPassword));
		curl_easy_setopt(curl_handle, CURLOPT_PROXYUSERPWD, strProxyAuth);
	}

	/* get it! */
	res = curl_easy_perform(curl_handle);
	
	char *effectiveUrl = NULL;
	curl_easy_getinfo(curl_handle, CURLINFO_EFFECTIVE_URL, &effectiveUrl);

	SplitUrl(effectiveUrl, host);

	/* cleanup curl stuff */
	curl_easy_cleanup(curl_handle);

	resp.SetData(chunk.memory, chunk.size);
	free(chunk.memory);

    if(res != CURLE_OK)
    {
        logFile.Write(LOG_ADVANCED, "HTTP POST failed. CURL error code = %d", res);
    }

	return (res == CURLE_OK ? 0 : 1);
}

/*******************************************************************
* Function  : DoGet()
* Parameters: url - The URL to which an HTTP GET is to be done
*             resp - The response object which will store the HTTP
*                    response received from the website
* Returns   : TRUE - GET succeeded (could have any HTTP errors,
*                    including 404 errors)
*             FALSE - The GET failed most likely because a 
*                    connection coul not be established
* Purpose   : To do an HTTP GET with a max of HTTP_MAX_RETRIES retries
*******************************************************************/
int HttpClient::DoGet(const char *url, HttpClientResponse &resp)
{
    int retVal = 0;

    for(int i = 0; i < HTTP_MAX_RETRIES; i++)
    {
        if( (retVal = DoGetInternal(url, resp)) == 0)
        {
            break;
        }
    }

    return retVal;
}

/*******************************************************************
* Function  : DoGetInternal()
* Parameters: url - The URL to which an HTTP GET is to be done
*             resp - The response object which will store the HTTP
*                    response received from the website
* Returns   : TRUE - GET succeeded (could have any HTTP errors,
*                    including 404 errors)
*             FALSE - The GET failed most likely because a 
*                    connection coul not be established
* Purpose   : To do an HTTP GET
*******************************************************************/
int HttpClient::DoGetInternal(const char *url, HttpClientResponse &resp)
{
	CURLcode res;
	MemoryStruct chunk;
	char strProxy[255];
	char strProxyAuth[255];
	
	chunk.memory = NULL; /* we expect realloc(NULL, size) to work */
	chunk.size   = 0;    /* no data at this point */
	chunk.session = session;

	session->GetCookies(this);

	/* specify URL to get */
	curl_easy_setopt(curl_handle, CURLOPT_URL, url);

	curl_easy_setopt(curl_handle, CURLOPT_FILE, static_cast<void *>(&chunk));
	curl_easy_setopt(curl_handle, CURLOPT_WRITEHEADER, static_cast<void *>(&chunk));
	curl_easy_setopt(curl_handle, CURLOPT_HTTPHEADER, headers);

	if(strnicmp(url, "https://", 8) == 0)
	{
		if(bUseHttpsProxy)
		{
			sprintf(strProxy, "%s:%d", httpsProxyHost, httpsProxyPort);
			curl_easy_setopt(curl_handle, CURLOPT_PROXY, strProxy);
		}
	}
	else if(bUseHttpProxy)
	{
		sprintf(strProxy, "%s:%d", httpProxyHost, httpProxyPort);
		curl_easy_setopt(curl_handle, CURLOPT_PROXY, strProxy);
	}

	if(settings.m_bProxyAuth == TRUE)
	{
		sprintf(strProxyAuth, "%s:%s", LPCTSTR(settings.m_strProxyAuthUsername), LPCTSTR(settings.m_strProxyAuthPassword));
		curl_easy_setopt(curl_handle, CURLOPT_PROXYUSERPWD, strProxyAuth);
	}
	
	/* get it! */
	res = curl_easy_perform(curl_handle);
	
	char *effectiveUrl = NULL;
	curl_easy_getinfo(curl_handle, CURLINFO_EFFECTIVE_URL, &effectiveUrl);
	SplitUrl(effectiveUrl, host);

	
	char *conttype = NULL;
	curl_easy_getinfo(curl_handle, CURLINFO_CONTENT_TYPE, &conttype);
	logFile.Write(LOG_ADVANCED, "Content-Type = %s", conttype);
    
	resp.SetData(chunk.memory, chunk.size);
	resp.SetContentType(conttype);

	free(chunk.memory);

    if(res != CURLE_OK)
    {
        logFile.Write(LOG_ADVANCED, "HTTP GET failed. CURL error code = %d", res);
    }

	return (res == CURLE_OK ? 0 : 1);
}

/*******************************************************************
* Function  : Reset()
* Parameters: -
* Returns   : -
* Purpose   : To partially cleanup the object so that it can be
*             reused for more requests. Note that the proxy settings
*             are not reset.
*******************************************************************/
void HttpClient::Reset()
{
	if(pResponsePage != NULL)
	{
		free(pResponsePage);
	}

	pResponsePage = NULL;
	responsePageLen = 0;
	
	curl_slist_free_all(headers);
	headers = NULL;

	host[0]	= '\0';
}

/*******************************************************************
* Function  : GetHost()
* Parameters: host - The object to which the host value is to be
*                    copied
* Returns   : The host value via a parameter
* Purpose   : To retrieve the host value. This is required because
*             HTTP redirects could have resulted in the host to 
*             change to a new one. All future requests will have to
*             be given to the new host only.
*******************************************************************/
void HttpClient::GetHost(CStdString &host)
{
	host = this->host;
}

/*******************************************************************
* Function  : SplitUrl()
* Parameters: url - The URL to be split into its components
*             host - The web server host name
* Returns   : 0 - Successful split
*             ERROR_UNSUPPORTED_PROTOCOL - Unknown protocol
* Purpose   : To split the URL to its basic components so that
*             we know which website to connect to and what to fetch
*             from there.
*******************************************************************/
int HttpClient::SplitUrl(const char *url, char *host)
{
	int startPos;
	const char *ptr;

	// Check protocol
	if(strstr(url, "http://") == url)
	{
		startPos = 7;     // protocol field end
	}
	else if(strstr(url, "https://") == url)
	{
		startPos = 8;    // protocol field end
	}
	else
	{
		// Unsupported protocol
		return ERROR_UNSUPPORTED_PROTOCOL;
	}

	// Get web server name
	if((ptr = strchr(url + startPos, '/')) == NULL)
	{
		// Only host is specified without trailing "/"
		strcpy(host, url + startPos);
		return 0;
	}
	else
	{
		strncpy(host, url + startPos, ptr - (url + startPos));
		host[ptr - (url + startPos)] = '\0';
	}

	return 0;
}

