/*************************************************************************
*
* YahooPOPs!
*
* YahooPOPs! is an application that provides POP3 access to Yahoo! Mail.
* You do not need to enable POP3 access with Yahoo! to use this program.
* This program works within the license agreement of Yahoo! to provide
* you with POP3 access.
*
* YahooPOPs! emulates a POP3 mail server and should work with most of the
* email clients. Since YahooPOPs! is not a full-fledged POP3 mail server
* you might find it not working with all mail clients. Do keep us informed
* about such mail clients so that we can enhance YahooPOPs!
*
* Official Website: http://yahoopops.sourceforge.net
*
*-------------------------------------------------------------------------
*
* Name    :	LogFile.cpp
*
* Purpose : To maintain an activity log file
*
* Authors : Anuj Seth
* 			K.Shyam
*
* License : GNU Public License (Version 2 or later)
*
*-------------------------------------------------------------------------
*
* THIS SOFTWARE IS PROVIDED ``AS IS'' BY THE AUTHORS AND INDEPENDENT
* CONTRIBUTORS AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
* LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
* PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR 
* CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
* EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
* PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
* PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
* LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
* NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
* SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
*************************************************************************/
#include "stdafx.h"

#ifdef WIN32
#include "YahooPOPs.h"
#else // #ifdef WIN32
#include <string.h>
#include <unistd.h>
#endif

#include "LogFile.h"
#include "DataHolder.h"
#include <time.h>

extern CDataHolder settings;

LogFile::LogFile(const CStdString &filename)
{
#ifdef WIN32
	m_strLogFile = filename;
#else
	m_strLogFile.Format("%s/%s",CONFIG_FILE_LOC,LPCTSTR(filename));
#endif
	fp = fopen(m_strLogFile, "a");

	if(fp == NULL)
	{
		fp = fopen(m_strLogFile, "w");
	}
#ifndef WIN32
	if (fp == NULL)
	{
		fp = stderr;
	}
#endif
	fseek(fp, 0, SEEK_END);

	m_nFileSize = ftell(fp);
#ifndef WIN32
	/* i have to do all these shit things here becuase of that
	 * blasted g++ screws my happiness if i declare lock as a
	 * global variable in this file itself.it conflicts with
	 * iostream lock ! :-( */
	pthread_mutex_init(&lock,NULL);
#endif

}

LogFile::~LogFile()
{
	fclose(fp);
}

void LogFile::Write(const LOGTYPE logtype, const char *format, ...)
{
	if(settings.m_bCreateLog == FALSE)
	{
		/* Log file has been disabled */
		return;
	}
#ifdef WIN32
	lock.Lock();
#else
	pthread_mutex_lock(&lock);
#endif
	va_list list;
	va_start(list, format);

	CStdString formatStr;
	char strTime[80];

#ifdef WIN32
	char strDate[80];
	formatStr.Format("[%s %s] ", _strdate(strDate), _strtime(strTime));
	formatStr += format;
#else
	time_t timeVal=0;
	time(&timeVal);
    strcpy(strTime,ctime(&timeVal));
	if ( strTime[0] != '\0' )
	{
		strTime[strlen(strTime) -1 ] = '\0';
	}
	formatStr.Format("[ %s ] ",strTime);
	formatStr += format;
#endif
				
	char str[1024];
	vsprintf(str, LPCTSTR(formatStr), list);

	if(str[strlen(str) - 1] != '\n')
	{
		strcat(str, "\n");
	}
	else if(str[strlen(str) - 2] == '\r')
	{
		str[strlen(str) - 2] = '\n';
		str[strlen(str) - 1] = '\0';
	}

	m_nFileSize += strlen(str);

	if(logtype == LOG_BASIC)
	{
		/* If type is 0, then everything must be written */
		if(fp != NULL)
		{
			fprintf(fp, str);
		}
	}
	else if(settings.m_nLogLevel == LOG_ADVANCED && logtype == LOG_ADVANCED)
	{
		if(fp != NULL)
		{
			fprintf(fp, str);
		}
	}

	if(fp != NULL)
	{
		fflush(fp);
	}

	if(settings.m_bLimitLog && m_nFileSize > (settings.m_nLogSize * 1024))
	{
		/* Trim the file */
		Trim();
	}
#ifdef WIN32
	lock.Unlock();
#else
	pthread_mutex_unlock(&lock);
#endif
}

void LogFile::Trim()
{
	fclose(fp);
#ifdef WIN32
	unlink(m_strLogFile);
	fp = fopen(m_strLogFile, "at");
#else
	unlink(m_strLogFile);
	fp = fopen(m_strLogFile, "a+");
#endif
	m_nFileSize = 0;
}

