/*************************************************************************
*
* YahooPOPs!
*
* YahooPOPs! is an application that provides POP3 access to Yahoo! Mail.
* You do not need to enable POP3 access with Yahoo! to use this program.
* This program works within the license agreement of Yahoo! to provide
* you with POP3 access.
*
* YahooPOPs! emulates a POP3 mail server and should work with most of the
* email clients. Since YahooPOPs! is not a full-fledged POP3 mail server
* you might find it not working with all mail clients. Do keep us informed
* about such mail clients so that we can enhance YahooPOPs!
*
* Official Website: http://yahoopops.sourceforge.net
*
*-------------------------------------------------------------------------
*
* Name    :	ServiceClient.cpp
*
* Purpose : Class to service a new POP3 connection
*           - Thread-safe implementation
*
* Authors : Anuj Seth & K Shyam
*
* License : GNU Public License (Version 2 or later)
*
*-------------------------------------------------------------------------
*
* THIS SOFTWARE IS PROVIDED ``AS IS'' BY THE AUTHORS AND INDEPENDENT
* CONTRIBUTORS AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
* LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
* PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR 
* CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
* EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
* PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
* PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
* LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
* NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
* SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
*************************************************************************/
#include "stdafx.h"

#ifdef WIN32
#include <winsock.h>
#include "resource.h"
#include "ProgressWnd.h"
#include "mainfrm.h"
#else // #ifdef WIN32
#include <errno.h>
#include <string.h>
#include "YAHOOPOPS.h"
#endif

#include "ServiceClient.h"
#include "DataHolder.h"
#include "WebBrowser.h"
#include "LogFile.h"
#include "YahooSessionCache.h"

extern CDataHolder		 settings;
extern LogFile			 logFile;
extern YahooSessionCache sessionCache;

#ifdef WIN32
extern CMainFrame		 *gp_MainFrame;
#else
typedef void* (*THREAD_FUNCTION)(void *);
typedef void* THREAD_PARAMETER;
#define IDS_LOGIN_WRONG_PASSWORD "Incorrect Password"
#define IDS_LOGIN_WRONG_USERNAME "Invalid User Name"
#define IDS_INBOX_ERROR "Inbox Error"
#endif

/*******************************************************************
* Function  : ListenThread()
* Parameters: -
* Returns   : 0
* Purpose   : A thread that listens for POP3 client requests
*******************************************************************/
UINT ListenThread(void *lpParameter)
{
	struct sockaddr_in serv;
	struct sockaddr_in cli;
	int    listenfd = 0;
	int    sockfd   = 0;
	int    clilen   = 0;
	int    result   = 0;
	int    retVal   = 0;

#ifndef WIN32
	int howStarted = 0;
	( lpParameter != NULL ) ? ( howStarted =  *((int*)lpParameter) ) : ( howStarted = 0 );
#endif
	if((listenfd = socket(AF_INET, SOCK_STREAM, 0)) < 0)
	{
#ifdef WIN32
		::AfxMessageBox(IDS_POP3_SOCKET_ERROR, MB_OK | MB_ICONSTOP);
		::PostMessage(::AfxGetMainWnd()->GetSafeHwnd(), WM_QUIT, 0, 0);
        AfxEndThread(0);
#else
		printf("Unable to create socket.Error reason = %s\n",strerror(errno));
		if ( howStarted == 1 ) 
		{
			pthread_exit((void*)NULL);
		}
		else
		{
			return(0);
		}
#endif
	}
	
	memset(&serv, 0, sizeof(serv));
	serv.sin_family 	 = AF_INET;
#ifdef WIN32
	serv.sin_addr.s_addr = inet_addr(settings.m_strNetworkIPAddr);
#else
	serv.sin_addr.s_addr = INADDR_ANY; 
#endif
	serv.sin_port		 = htons(settings.m_nNetworkPort);
	
	if (bind(listenfd,(struct sockaddr *) &serv, sizeof(serv)) < 0)
	{
#ifdef WIN32
		::AfxMessageBox(IDS_POP3_SOCKET_ERROR, MB_OK | MB_ICONSTOP);
		closesocket(listenfd);
        AfxEndThread(0);
#else
		printf("Unable to bind.Error reason = %s\n",strerror(errno));
		close(listenfd);
		if ( howStarted == 1 )
		{
			pthread_exit((void*)NULL);
		}
		else
		{
			return(0);
		}
#endif
	}

#ifndef WIN32
	/* we use this for shutdown purposes */
	listenThreadPort(&listenfd);
#endif

	
	if (listen(listenfd, 25) < 0)
	{
#ifdef WIN32
		::AfxMessageBox(IDS_POP3_SOCKET_ERROR, MB_OK | MB_ICONSTOP);
		closesocket(listenfd);
        AfxEndThread(0);
#else
		printf("Unable to listen.Error reason = %s\n",strerror(errno));
		close(listenfd);
		if ( howStarted == 1 ) 
		{
			pthread_exit((void*)NULL);
		}
		else
		{
			return(0);
		}
#endif

	}

	while(1)
	{
		clilen = sizeof(cli);

#ifdef WIN32
		if((sockfd = accept(listenfd,(struct sockaddr *) &cli,&clilen)) < 0)
#else
		if((sockfd = accept(listenfd,(struct sockaddr *) &cli,(socklen_t*)&clilen)) < 0)
#endif
		{
			break;
		}
		else
		{
			
			/* Edit SH. Dump the settings everytime if advanced mode on */
			if (settings.m_nLogLevel==LOG_ADVANCED)
				settings.DumpToLog();

#ifdef WIN32
			::AfxBeginThread(ServiceClientThread, (LPVOID)sockfd);
#else
			pthread_t threadId;
			pthread_create(&threadId,NULL,(THREAD_FUNCTION)ServiceClientThread,(THREAD_PARAMETER)&sockfd);
#endif
		}
	}
			
	// Exiting
#ifdef WIN32
	::PostMessage(::AfxGetMainWnd()->GetSafeHwnd(), WM_QUIT, 0, 0);
    AfxEndThread(0);
    return 0;
#else
	close(listenfd);
	if ( howStarted == 1 )
	{
		pthread_exit((void*)NULL);
	}
	else
	{
		return(0);
	}
#endif
}

/*******************************************************************
* Function  : ProgressThread
* Parameters: lpParameter -> The data object containing all the
*             information this thread required
* Returns   : -
* Purpose   : To show the progress of the service in a thread. A
*             thread is being used to ensure that the dialog keeps
*             painting itself without any problems.
*******************************************************************/
#ifdef WIN32
UINT ProgressThread(void *lpParameter)
{
	PROGRESS_DATA *data = reinterpret_cast<PROGRESS_DATA*>(lpParameter);

	if(settings.m_bQuietMode)
	{
		data->progress = NULL;
	}
	else
	{
		data->progress = new CProgressWnd(::AfxGetMainWnd(), "", TRUE);
		data->progress->SetProgressWnd(data->progress);
		data->progress->CenterWindow();
	}
	
	data->init.SetEvent();

	while(::WaitForSingleObject(data->destroy, 0) != WAIT_OBJECT_0)
	{
		if(data->progress != NULL)
		{
			data->progress->PeekAndPump();

			if(data->progress->Cancelled())
			{
				break;
			}
		}

		Sleep(500);
	}

	delete data;
	data = NULL;

    AfxEndThread(0);
    return 0;
}
#endif

/*******************************************************************
* Function  : ServiceClientThread
* Parameters: lpParameter -> The client socket
* Returns   : Return value from service.Execute()
* Purpose   : To service a POP3 clients request. This is a thread
*             which is spawned by ListenThread()
*******************************************************************/
UINT ServiceClientThread(void *lpParameter)
{
#ifndef WIN32
	int sock = 0;
#endif
	int retVal;
	try
	{
#ifdef WIN32
		SOCKET sock = (reinterpret_cast<SOCKET>(lpParameter));
		ServiceClient service(sock);
#else
		sock = *((int*)lpParameter);
		ServiceClient service(&sock);
#endif
		retVal = service.Execute();
	}
	catch(...)
	{
		/* 
		 * This catch will catch only C++ exceptions.
		 * I don't expect it to catch any C runtime
		 * library exceptions which could be caused
		 * due to the bugs in my code :(
		 */
		retVal = 1;
	}
#ifdef WIN32
	AfxEndThread(retVal);
#else
	close(sock);
	sleep(1);
	pthread_exit(NULL);	
#endif
    return 0;
}

/*******************************************************************
* Function  : ServiceClient() constructor
* Parameters: sock -> POP3 Client socket
* Returns   : -
* Purpose   : Initialize the ServiceClient object and send the
*             "WELCOME" greeting to the POP3 client.
*******************************************************************/
#ifdef WIN32
ServiceClient::ServiceClient(SOCKET sock)
#else
	//added by shyam on 28-05-04	
	/* ok why does the parameter & the class member have the same name ? */
	//end added by shyam on 28-05-04	
ServiceClient::ServiceClient(SOCKET *socket)
#endif
{

#ifdef WIN32
	this->sock = sock;
	str.LoadString(IDS_POP3_READY);
    send(sock, str, str.GetLength(), 0);
#else
	this->sock = *socket;
	char *temp_str = "+OK POP3 YahooPOPs! proxy ready\r\n";
    send(*socket,temp_str,strlen(temp_str), 0);
#endif
}

/*******************************************************************
* Function  : ServiceClient() destructor
* Parameters: -
* Returns   : -
* Purpose   : Object cleanup
*******************************************************************/
ServiceClient::~ServiceClient()
{
#ifdef WIN32
	closesocket(sock);
#else
	close(sock);
#endif
}

/*******************************************************************
* Function  : ReadUser()
* Parameters: -
* Returns   : 0 - Success
*             1 - Error
* Purpose   : To read the "USER <user>" data from the socket
*******************************************************************/
int ServiceClient::ReadUser()
{
	memset(buf,'\0', 1023);
	buflen = recv(sock, buf, 1023, 0);

	if(buflen <= 0)
	{
		return 1;
	}

	if( (ptr = strchr(buf, ' ')) != NULL && strnicmp(buf, "USER", 4) == 0)
	{
		/* Fix provided by glibdud@users.sourceforge.net */
		sscanf(buf, "%*4s %s\r\n", username);
#ifdef WIN32
		str.LoadString(IDS_POP3_USER_OK);
        send(sock, str, str.GetLength(), 0);
#else
		char *temp_str = "+OK User name accepted, password please\r\n";
        send(sock,temp_str,strlen(temp_str), 0);
#endif
	}
	else
	{
#ifdef WIN32
		str.LoadString(IDS_POP3_UNKNOWN_AUTH);
        send(sock, str, str.GetLength(), 0);
#else
		char *temp_str ="-ERR Unknown AUTHORIZATION state command\r\n";
        send(sock,temp_str,strlen(temp_str), 0);
#endif
		return 2;
	}

	return 0;
}

/*******************************************************************
* Function  : ReadPass()
* Parameters: -
* Returns   : 0 - Success
*             1 - Error
* Purpose   : To read the "PASS <password>" data from the socket
*******************************************************************/
int ServiceClient::ReadPass()
{
	memset(buf,'\0', 1023);
	buflen = recv(sock, buf, 1023, 0);

	if( (ptr = strchr(buf, ' ')) != NULL && strnicmp(buf, "PASS", 4) == 0)
	{
		/*The original sscanf is replaced to fix
		  the 'space in password problem' (by ltd)*/
		char* ptr=strstr(buf,"\r\n");
		*ptr=0;
		ptr=strchr(buf,' ');
		while (*(++ptr)==' ');
		strcpy(password,ptr);
	}
	else
	{
#ifdef WIN32
		str.LoadString(IDS_POP3_UNKNOWN_AUTH);
        send(sock, str, str.GetLength(), 0);
		closesocket(sock);
#else
		char *temp_str="-ERR Unknown AUTHORIZATION state command\r\n";
        send(sock,temp_str,strlen(temp_str), 0);
		close(sock);
#endif
		return 1;
	}

	return 0;
}

/*******************************************************************
* Function  : Execute()
* Parameters: -
* Returns   : 0 - Success
*             1 - Error
* Purpose   : The main function which processes the POP3 clients
*             requests.
*******************************************************************/
int ServiceClient::Execute()
{
	int i;
	CStdString tooltip;

	do
	{
		i = ReadUser();

		if(i == 1)
		{
			return 1;
		}

	}while(i != 0);

	logFile.Write(LOG_BASIC, "Starting POP3 session for \"%s\".", username);
	
#ifdef WIN32
	tooltip.Format("[%s] Starting POP3 session", username);

	PROGRESS_DATA *data = new PROGRESS_DATA;

	::AfxBeginThread(ProgressThread, data);
	::WaitForSingleObject(data->init, INFINITE);

	CString caption;
	caption.Format("Email download status for %s", username);

	if(data->progress != NULL)
	{
		data->progress->SetCaption(caption);
		data->progress->SetRange(0, 100);
		data->progress->SetText("Waiting for password...");
	}

	if(ReadPass() != 0)
	{
		data->destroy.SetEvent();
		return 1;
	}

	logFile.Write(LOG_ADVANCED, "<%s> Password received from client.", username);

	if(data->progress != NULL)
	{
		data->progress->SetText("Connecting to Yahoo! Mail...");
	}
#else
	if(ReadPass() != 0)
	{
	    logFile.Write(LOG_ADVANCED,"Unable to read password\n");
		return 1;
	}

	logFile.Write(LOG_ADVANCED, "<%s> Password received from client.\n", username);
#endif

	int retVal = browser.ConnectToYahoo(username, password);

#ifdef WIN32
	if(data == NULL || (data->progress != NULL && data->progress->Cancelled()))
	{
		/* User clicked on the Cancel button */
		closesocket(sock);

		if(data != NULL)
		{
			data->destroy.SetEvent();
		}

		return 0;
	}
#endif

	switch(retVal)
	{
	case WEB_LOGIN_OK:
		/* Perfect! Now we proceed with the actual work */
		logFile.Write(LOG_BASIC, "<%s> Login to Yahoo Mail succeeded.", username);
		tooltip.Format("[%s] Login to Yahoo Mail succeeded", username);
		break;

	case WEB_LOGIN_FAILED:
#ifdef WIN32
		closesocket(sock);
		tooltip.Format("[%s] Could not connect to Yahoo!", username);
		data->destroy.SetEvent();
#else
		close(sock);
#endif
		logFile.Write(LOG_BASIC, "<%s> Login to Yahoo Mail failed.", username);
        browser.DeleteSession();
		return 1;
		
	case WEB_UNKNOWN_ERROR:
#ifdef WIN32
		closesocket(sock);
		tooltip.Format("[%s] Could not connect to Yahoo!. Unknown Error (Raise bug report and attach yahoopops.log)", username);
		data->destroy.SetEvent();
#else
		close(sock);
#endif
		logFile.Write(LOG_BASIC, "<%s> Login to Yahoo Mail failed (Unknown error).", username);
        browser.DeleteSession();
		return 1;

	case WEB_WRONG_PASSWORD:
		SendError(IDS_LOGIN_WRONG_PASSWORD);
		logFile.Write(LOG_BASIC, "<%s> Login to Yahoo Mail failed due to an incorrect password.", username);
#ifdef WIN32
		tooltip.Format("[%s] Login to Yahoo Mail failed due to an incorrect password", username);
		data->destroy.SetEvent();
#endif
        browser.DeleteSession();
		return 1;

	case WEB_WRONG_USERNAME:
		SendError(IDS_LOGIN_WRONG_USERNAME);
		logFile.Write(LOG_BASIC, "<%s> Login to Yahoo Mail failed due to an incorrect logid id.", username);
		tooltip.Format("[%s] Login to Yahoo Mail failed due to an incorrect logid id", username);
#ifdef WIN32
		data->destroy.SetEvent();
#endif
        browser.DeleteSession();
		return 1;

	default: /* Treat as unknown error! */
#ifdef WIN32
		closesocket(sock);
#else
		close(sock);
#endif
		logFile.Write(LOG_BASIC, "<%s> Login to Yahoo Mail failed due to a bug in YahooPOPs!.", username);
#ifdef WIN32
		tooltip.Format("[%s] Login to Yahoo Mail failed due to a bug in YahooPOPs! Raise bug report and attach yahoopops.log", username);
		data->destroy.SetEvent();
#endif
        browser.DeleteSession();
		return 1;
	}

#ifdef WIN32
	gp_MainFrame->m_TrayIcon.SetTooltipText(LPCTSTR(tooltip));
#endif

	//EDIT: Shane .h : Save the session even if we aren't using SMTP
    //if(settings.m_bEnableSmtp)
    //{
	//Session is already saved in connectToYahoo (if it was successful)
	//browser.SaveSession();
    //}

#ifdef WIN32
	if(data->progress != NULL)
	{
		data->progress->SetText("Analyzing the Inbox folder for emails...");
	}
#endif

    str = "+OK Mailbox open\r\n";
	send(sock, str, str.GetLength(), 0);

	if(settings.m_bInbox)
	{
		logFile.Write(LOG_BASIC, "<%s> Analyzing the INBOX folder", username);

#ifdef WIN32
		tooltip.Format("[%s] Analyzing the INBOX folder", username);
		gp_MainFrame->m_TrayIcon.SetTooltipText(LPCTSTR(tooltip));
#endif

        retVal = browser.FetchMailboxList(yahooMbox[MB_INBOX]);
		
		logFile.Write(LOG_BASIC, "<%s> INBOX folder analyzed", username);

#ifdef WIN32	
		if(data == NULL || (data->progress != NULL && data->progress->Cancelled()))
		{
			/* User clicked on the Cancel button */
			closesocket(sock);
			
			logFile.Write(LOG_ADVANCED, "<%s> Cancel event received", username);
			
			if(data != NULL)
			{
				data->destroy.SetEvent();
			}
			
			gp_MainFrame->m_TrayIcon.SetTooltipText("YahooPOPs!");
			return 0;
		}
#endif

		switch(retVal)
		{
		case WEB_INBOX_FETCH_OK:
			/* You've got Mail! */
			break;
			
		case WEB_INBOX_EMPTY:
			/* You've got no mail :( */	
			break;
			
		case WEB_INBOX_FETCH_FAILED:
		case WEB_UNKNOWN_ERROR:
        default:

#ifdef WIN32
			gp_MainFrame->m_TrayIcon.SetTooltipText("YahooPOPs!");
			SendError(IDS_INBOX_ERROR);
			data->destroy.SetEvent();
#else
			SendError(IDS_INBOX_ERROR);
#endif
			return 1;
		}
	}

	if(settings.m_bBulkMail)
	{
#ifdef WIN32
		if(data->progress != NULL)
		{
			data->progress->SetText("Analyzing the Bulk Mail folder for emails...");
		}

        tooltip.Format("[%s] Analyzing the BulkMail folder", username);
		gp_MainFrame->m_TrayIcon.SetTooltipText(LPCTSTR(tooltip));

#endif

		logFile.Write(LOG_BASIC, "<%s> Analyzing the BulkMail folder", username);
		retVal = browser.FetchMailboxList(yahooMbox[MB_BULKMAIL]);
		logFile.Write(LOG_BASIC, "<%s> BulkMail folder analyzed", username);
	}

#ifdef WIN32	
	if(data == NULL || (data->progress != NULL && data->progress->Cancelled()))
	{
		/* User clicked on the Cancel button */
		closesocket(sock);

		logFile.Write(LOG_ADVANCED, "<%s> Cancel event received", username);
	
		if(data != NULL)
		{
			data->destroy.SetEvent();
		}

		gp_MainFrame->m_TrayIcon.SetTooltipText("YahooPOPs!");
		return 0;
	}
#endif

    /* Analyze all the user defined folders */
	int prevLoc = 0;

	while(1)
	{
		CStdString folderName;

		int i = settings.m_strFolderList.Find("|", prevLoc);

		if(i == -1)
		{
			folderName = settings.m_strFolderList.Mid(prevLoc);

			if(folderName.IsEmpty())
			{
				break;
			}
		}
		else
		{
			folderName = settings.m_strFolderList.Mid(prevLoc, i - prevLoc);
		}

#ifdef WIN32		
		if(data->progress != NULL)
		{
			str.Format("Analyzing the %s folder for emails...", LPCTSTR(folderName));
			data->progress->SetText(str);
		}
#endif

		logFile.Write(LOG_BASIC, "<%s> Analyzing the %s folder", username, LPCTSTR(folderName));

#ifdef WIN32
		tooltip.Format("[%s] Analyzing the %s folder", username, LPCTSTR(folderName));
		gp_MainFrame->m_TrayIcon.SetTooltipText(LPCTSTR(tooltip));
#endif

        retVal = browser.FetchMailboxList(folderName);

		logFile.Write(LOG_BASIC, "<%s> %s folder analyzed", username, LPCTSTR(folderName));

#ifdef WIN32
		if(data == NULL || (data->progress != NULL && data->progress->Cancelled()))
		{
			/* User clicked on the Cancel button */
			closesocket(sock);

			logFile.Write(LOG_ADVANCED, "<%s> Cancel event received", username);
		
			if(data != NULL)
			{
				data->destroy.SetEvent();
			}

			gp_MainFrame->m_TrayIcon.SetTooltipText("YahooPOPs!");
			return 0;
		}
#endif

		prevLoc = i + 1;

		if(i == -1)
		{
			break;
		}
	}

	/* Optional download of all emails may be turned of
	   if client doesn't care about exact message size 
	   (experimental feature by ltd) */
	if (!settings.m_bIgnoreSize) {
	/* 
	 * Download all emails because most of the POP3 commands
	 * that will be received from any POP3 client would
	 * require knowledge about the email size, etc. So it
	 * makes sense to download them.
	 */

	logFile.Write(LOG_BASIC, "<%s> Found %d emails", username, browser.GetMsgCount());

#ifdef WIN32	
	if(data->progress != NULL)
	{
		data->progress->SetRange(0, browser.GetMsgCount());
	}
#endif

	if(browser.GetMsgCount() != 0)
	{
#ifdef WIN32
		tooltip.Format("[%s] Downloading emails", username);
        gp_MainFrame->m_TrayIcon.SetTooltipText(tooltip);
			retVal = browser.DownloadAllEmails(data->progress);
#else
			retVal = browser.DownloadAllEmails();
#endif
	}

#ifdef WIN32
	if(data == NULL || (data->progress != NULL && data->progress->Cancelled()))
	{
		/* User clicked on the Cancel button */
		closesocket(sock);

		logFile.Write(LOG_ADVANCED, "<%s> Cancel event received", username);

		if(data != NULL)
		{
			data->destroy.SetEvent();
		}

		gp_MainFrame->m_TrayIcon.SetTooltipText("YahooPOPs!");
		return 0;
	}

	if(data->progress != NULL)
	{
		data->progress->SetText("Servicing your mail clients requests...");
	}
#endif
	}

	CStdString progressStr;

#ifdef WIN32
	tooltip.Format("[%s] Servicing email application requests", username);
	gp_MainFrame->m_TrayIcon.SetTooltipText(LPCTSTR(tooltip));
#endif

	while(TRUE)
	{
		memset(buf, '\0', 1023);
		buflen = recv(sock, buf, 1023, 0);

		if(buflen <= 0)
		{
			/* Socket has been closed */
			break;
		}

		logFile.Write(LOG_ADVANCED, "<%s> Command received from email client: %s", username, buf);

#ifdef WIN32		
		if(data == NULL || (data->progress != NULL && data->progress->Cancelled()))
		{
			/* User clicked on the Cancel button */
			closesocket(sock);

			logFile.Write(LOG_ADVANCED, "<%s> Cancel event received", username);

			if(data != NULL)
			{
				data->destroy.SetEvent();
			}
	
			gp_MainFrame->m_TrayIcon.SetTooltipText("YahooPOPs!");
			return 0;
		}

		if(data->progress != NULL)
		{
			progressStr.Format("Request: %s", buf);
			data->progress->SetText(progressStr);
		}
#endif
		if(strnicmp(buf, "STAT", 4) == 0)
		{
			retVal = ServiceSTAT();
		}
		else if(strnicmp(buf, "LIST", 4) == 0)
		{
			retVal = ServiceLIST(buf);
		}
		else if(strnicmp(buf, "RETR", 4) == 0)
		{
			retVal = ServiceRETR(buf);
		}
		else if(strnicmp(buf, "DELE", 4) == 0)
		{
			retVal = ServiceDELE(buf);
		}
		else if(strnicmp(buf, "NOOP", 4) == 0)
		{
			retVal = ServiceNOOP();
		}
		else if(strnicmp(buf, "RSET", 4) == 0)
		{
			retVal = ServiceRSET();
		}
		else if(strnicmp(buf, "QUIT", 4) == 0)
		{
			/* Mark unread */
			if(settings.m_bLeaveUnread)
			{
#ifdef WIN32
				if(data->progress != NULL)
				{
					progressStr.Format("Marking emails as unread on server...");
					data->progress->SetText(progressStr);
				}
#endif
				browser.MarkUnread(yahooMbox[MB_INBOX]);
				browser.MarkUnread(yahooMbox[MB_BULKMAIL]);

				prevLoc = 0;

				while(1)
				{
					CStdString folderName;
					int i = settings.m_strFolderList.Find("|", prevLoc);

					if(i == -1)
					{
						folderName = settings.m_strFolderList.Mid(prevLoc);

						if(folderName.IsEmpty())
						{
							break;
						}
					}
					else
					{
						folderName = settings.m_strFolderList.Mid(prevLoc, i - prevLoc);
					}

					prevLoc = i + 1;

					browser.MarkUnread(folderName);
					
					if(i == -1)
					{
						break;
					}
				}
			}
#ifdef WIN32
			if(data->progress != NULL && browser.AnyEmailsToDelete())
			{
				progressStr.Format("Deleting emails from Yahoo...");
				data->progress->SetText(progressStr);
			}
#endif
			retVal = ServiceQUIT();

			/* Empty Trash */
			if(settings.m_bEmptyTrash)
			{
#ifdef WIN32
				if(data->progress != NULL)
				{
					progressStr.Format("Emptying trash...");
					data->progress->SetText(progressStr);
				}
#endif
				browser.EmptyTrash();
			}

			if(settings.m_bEmptyBulkMail)
			{
#ifdef WIN32
				if(data->progress != NULL)
				{
					progressStr.Format("Emptying bulk mail...");
					data->progress->SetText(progressStr);
				}
#endif
				browser.EmptyBulkMail();
			}
			break;
		}
		else if(strnicmp(buf, "TOP", 3) == 0)
		{
			retVal = ServiceTOP(buf);
		}
		else if(strnicmp(buf, "UIDL", 4) == 0)
		{
			retVal = ServiceUIDL();
		}
		else
		{
			/* Catch all */
			str = "-ERR Unknown command\r\n";
			send(sock, str, str.GetLength(), 0);
		}
	}

	/* The following call to save session might seem redudant,
	 * but it actually updates the last access time for the 
	 * session.
	 */
    
	//EDIT: Shane. H - We don't want to update the last access time anymore
	//if(settings.m_bEnableSmtp)
    //{
	//browser.SaveSession();
	//}

#ifdef WIN32
	closesocket(sock);
	gp_MainFrame->m_TrayIcon.SetTooltipText("YahooPOPs!");
	data->destroy.SetEvent();
#else
	int sockRetVal = close(sock);
	printf("Socket closed.Error = %s Retval = %d\n",strerror(errno),sockRetVal);
#endif

    return 0;
}

/*******************************************************************
* Function  : SendError()
* Parameters: errorCode - The error message to send to the client
* Returns   : -
* Purpose   : To send an error message to the POP3 client
*******************************************************************/
#ifdef WIN32
void ServiceClient::SendError(int errorCode)
#else
void ServiceClient::SendError(char *str)
#endif
{
#ifdef WIN32
	str.LoadString(errorCode);
	send(sock, str, str.GetLength(), 0);
#else
	send(sock, str, strlen(str), 0);
#endif
	/* Close the socket as we cannot proceed any further */
#ifdef WIN32
	closesocket(sock);
#else
	close(sock);
#endif
}

/*******************************************************************
* Function  : ServiceSTAT()
* Parameters: -
* Returns   : 0
* Purpose   : To send the client the response to the STAT command
*******************************************************************/
int ServiceClient::ServiceSTAT()
{
	int totalMessages = browser.GetMsgCount();
	int undeletedMsgs = 0;
	int msgSize       = 0;
	Email *email      = NULL;

	for(int i = 0; i < totalMessages; i++)
	{
		email = browser.GetEmail(i + 1);

		if(email != NULL)
		{
			if(!email->IsDeleted())
			{
				undeletedMsgs++;
				msgSize += email->GetSize();
			}
		}
	}

	str.Format("+OK %d %d\r\n", undeletedMsgs, msgSize);
	send(sock, str, str.GetLength(), 0);

	logFile.Write(LOG_ADVANCED, "<%s> Response sent: %s", username, LPCTSTR(str));

	return 0;
}

/*******************************************************************
* Function  : ServiceLIST()
* Parameters: buf - The request line from the client
* Returns   : 0
* Purpose   : To send the client the response to the LIST command
*******************************************************************/
int ServiceClient::ServiceLIST(char *buf)
{
	int msgNumber = -1;
	Email *email  = NULL;

	sscanf(buf, "%*4s %d\r\n", &msgNumber);

	if(msgNumber == -1)
	{
		/* Return entire list */
		str = "+OK scan listing follows\r\n";
		send(sock, str, str.GetLength(), 0);

		int totalMessages = browser.GetMsgCount();
		int undeletedMsgs = 0;

		for(int i = 0; i < totalMessages; i++)
		{
			email = browser.GetEmail(i + 1);

			if(email != NULL)
			{
				if(!email->IsDeleted())
				{
					undeletedMsgs++;
					str.Format("%d %d\r\n", undeletedMsgs, email->GetSize());
					send(sock, str, str.GetLength(), 0);

					logFile.Write(LOG_ADVANCED, "<%s> Response sent: %s", username, LPCTSTR(str));
				}
			}
		}

		str = ".\r\n";
		send(sock, str, str.GetLength(), 0);
		logFile.Write(LOG_ADVANCED, "<%s> Response sent: %s", username, LPCTSTR(str));
	}
	else
	{
		/* Return details for specified message only */
		email = browser.GetEmail(msgNumber);

		if(email != NULL && !email->IsDeleted() && (email->IsDownloaded() || settings.m_bIgnoreSize))
		{
			str.Format("+OK %d %d\r\n", msgNumber, email->GetSize());
			send(sock, str, str.GetLength(), 0);
			logFile.Write(LOG_ADVANCED, "<%s> Response sent: %s", username, LPCTSTR(str));
		}
		else
		{
			str.Format("-ERR no such message OR message marked as deleted OR message not downloaded\r\n");
			send(sock, str, str.GetLength(), 0);
			logFile.Write(LOG_ADVANCED, "<%s> Response sent: %s", username, LPCTSTR(str));
		}
	}

	return 0;
}

/*******************************************************************
* Function  : ServiceRETR()
* Parameters: buf - The request line from the client
* Returns   : 0
* Purpose   : To send the client the response to the RETR command
*******************************************************************/
int ServiceClient::ServiceRETR(char *buf)
{
	int msgNumber = -1;

	sscanf(buf, "%*4s %d\r\n", &msgNumber);

	Email *email = browser.GetEmail(msgNumber);

	if(email == NULL)
	{
		str = "-ERR no such message\r\n";
		send(sock, str, str.GetLength(), 0);
		logFile.Write(LOG_ADVANCED, "<%s> Response sent: %s", username, LPCTSTR(str));
	}
	else if(email->IsDeleted())
	{
		str.Format("-ERR message %d has been marked as deleted\r\n", msgNumber);
		send(sock, str, str.GetLength(), 0);
		logFile.Write(LOG_ADVANCED, "<%s> Response sent: %s", username, LPCTSTR(str));
	}
	else
	{
		/* RETR command requires the message to be downloaded */
		if (!email->IsDownloaded() && settings.m_bIgnoreSize)
			browser.DownloadEmail(msgNumber);

 		if(email->IsDownloaded())
		{
			email->SendMessage(sock);
			logFile.Write(LOG_ADVANCED, "<%s> Response sent: [Email sent to client]", username);
		}
		else
		{
			str.Format("-ERR message %d could not be downloaded\r\n", msgNumber);
			send(sock, str, str.GetLength(), 0);
			logFile.Write(LOG_ADVANCED, "<%s> Response sent: %s", username, LPCTSTR(str));
		}
	}
	
	return 0;
}

/*******************************************************************
* Function  : ServiceDELE()
* Parameters: buf - The request line from the client
* Returns   : 0
* Purpose   : To send the client the response to the DELE command
*******************************************************************/
int ServiceClient::ServiceDELE(char *buf)
{
	int msgNumber = -1;

	sscanf(buf, "%*4s %d\r\n", &msgNumber);
	Email *email = browser.GetEmail(msgNumber);

	if(email == NULL)
	{
		str = "-ERR No such message\r\n";
	}
	else
	{
		if(email->IsDeleted())
		{
			str.Format("-ERR message %d already deleted\r\n", msgNumber);
		}
		else
		{
			email->SetDeleted(TRUE);
			str.Format("+OK message %d deleted\r\n", msgNumber);
		}
	}

	send(sock, str, str.GetLength(), 0);
	logFile.Write(LOG_ADVANCED, "<%s> Response sent: %s", username, LPCTSTR(str));

	return 0;
}

/*******************************************************************
* Function  : ServiceNOOP()
* Parameters: -
* Returns   : 0
* Purpose   : To send the client the response to the NOOP command
*******************************************************************/
int ServiceClient::ServiceNOOP()
{
	str = "+OK\r\n";
	send(sock, str, str.GetLength(), 0);
	logFile.Write(LOG_ADVANCED, "<%s> Response sent: %s", username, LPCTSTR(str));

	return 0;
}

/*******************************************************************
* Function  : ServiceRSET()
* Parameters: -
* Returns   : 0
* Purpose   : To send the client the response to the RSET command
*******************************************************************/
int ServiceClient::ServiceRSET()
{
	Email *email;
	int msgCount = browser.GetMsgCount();

	for(int i = 0; i < msgCount; i++)
	{
		email = browser.GetEmail(i + 1);

		if(email != NULL)
		{
			email->SetDeleted(FALSE);
		}
	}

	str.Format("+OK YahooPOPs! now has %d messages for you\r\n", msgCount);
	send(sock, str, str.GetLength(), 0);
	logFile.Write(LOG_ADVANCED, "<%s> Response sent: %s", username, LPCTSTR(str));

	return 0;
}

/*******************************************************************
* Function  : ServiceQUIT()
* Parameters: -
* Returns   : 0
* Purpose   : To send the client the response to the QUIT command
*******************************************************************/
int ServiceClient::ServiceQUIT()
{
	str = "+OK YahooPOPs! signing off. Hope you enjoyed this program.\r\n";
	send(sock, str, str.GetLength(), 0);
	logFile.Write(LOG_ADVANCED, "<%s> Response sent: %s", username, LPCTSTR(str));

    if(!browser.AnyEmailsToDelete())
    {
        return 0;
    }

	logFile.Write(LOG_ADVANCED, "<%s> Deleting marked emails", username);

	browser.DeleteEmails(yahooMbox[MB_INBOX]);
	browser.DeleteEmails(yahooMbox[MB_BULKMAIL]);

	int prevLoc = 0;

	while(1)
	{
		CStdString folderName;

		int i = settings.m_strFolderList.Find("|", prevLoc);

		if(i == -1)
		{
			folderName = settings.m_strFolderList.Mid(prevLoc);

			if(folderName.IsEmpty())
			{
				break;
			}
		}
		else
		{
			folderName = settings.m_strFolderList.Mid(prevLoc, i - prevLoc);
		}

		browser.DeleteEmails(folderName);

		prevLoc = i + 1;

		if(i == -1)
		{
			break;
		}
	}

	/* 
	 * The thread will automatically be closed; because we
	 * have closed the socket. The recv() call will fail
	 * resulting in the thread to be closed.
	 */

	return 0;
}

/*******************************************************************
* Function  : ServiceTOP()
* Parameters: buf - The request line from the client
* Returns   : 0
* Purpose   : To send the client the response to the TOP command
*******************************************************************/
int ServiceClient::ServiceTOP(char *buf)
{
	int msgNumber = -1;
	int lines	  = -1;

	sscanf(buf, "%*3s %d %d\r\n", &msgNumber, &lines);

	Email *email = browser.GetEmail(msgNumber);

	if(email == NULL)
	{
		str = "-ERR no such message\r\n";
		send(sock, str, str.GetLength(), 0);
		logFile.Write(LOG_ADVANCED, "<%s> Response sent: %s", username, LPCTSTR(str));
	}
	else if(lines < 0)
	{
		str = "-ERR invalid number of lines\r\n";
		send(sock, str, str.GetLength(), 0);
		logFile.Write(LOG_ADVANCED, "<%s> Response sent: %s", username, LPCTSTR(str));
	}
	else if(email->IsDeleted())
	{
		str.Format("-ERR message %d has been marked as deleted\r\n", msgNumber);
		send(sock, str, str.GetLength(), 0);
		logFile.Write(LOG_ADVANCED, "<%s> Response sent: %s", username, LPCTSTR(str));
	}
	else
	{
		/* TOP command require the message to be downloaded 
		   or at least headers if requested number of lines is 0 */
		if (lines) {
			if (!email->IsDownloaded() && settings.m_bIgnoreSize)
				browser.DownloadEmail(msgNumber);
			if (!email->IsDownloaded())
		{
				str.Format("-ERR message %d could not be downloaded\r\n", msgNumber);
				send(sock, str, str.GetLength(), 0);
				logFile.Write(LOG_ADVANCED, "<%s> Response sent: %s", username, LPCTSTR(str));
				return 0;
		}
		} else {
			if (!email->IsHeaderDownloaded() && settings.m_bIgnoreSize)
				browser.DownloadEmail(msgNumber, TRUE);
			if (!email->IsHeaderDownloaded())
		{
				str.Format("-ERR headers of message %d could not be downloaded\r\n", msgNumber);
			send(sock, str, str.GetLength(), 0);
			logFile.Write(LOG_ADVANCED, "<%s> Response sent: %s", username, LPCTSTR(str));
				return 0;
		}
	}
	
		logFile.Write(LOG_ADVANCED, "<%s> Response sent: [%d lines of requested email]", username, lines);
		email->SendMessage(sock, lines);
	}
	
	return 0;
}

/*******************************************************************
* Function  : ServiceUIDL()
* Parameters: -
* Returns   : 0
* Purpose   : To send the client the response to the UIDL command
*******************************************************************/
int ServiceClient::ServiceUIDL()
{
    if(!settings.m_bEnableUIDL)
    {
        str = "-ERR UIDL support is currently disabled in YahooPOPs";
		send(sock, str, str.GetLength(), 0);
		logFile.Write(LOG_ADVANCED, "<%s> Response sent: %s", username, LPCTSTR(str));
        return 0;
    }

	int msgNumber = -1;
	Email *email  = NULL;

	sscanf(buf, "%*4s %d\r\n", &msgNumber);

	if(msgNumber == -1)
	{
		/* Return entire list */
		str = "+OK scan listing follows\r\n";
		send(sock, str, str.GetLength(), 0);

		int totalMessages = browser.GetMsgCount();
		int undeletedMsgs = 0;

		for(int i = 0; i < totalMessages; i++)
		{
			email = browser.GetEmail(i + 1);

			if(email != NULL)
			{
				if(!email->IsDeleted())
				{
					undeletedMsgs++;
					str.Format("%d %s\r\n", undeletedMsgs, email->GetMsgId());
					send(sock, str, str.GetLength(), 0);

					logFile.Write(LOG_ADVANCED, "<%s> Response sent: %s", username, LPCTSTR(str));
				}
			}
		}

		str = ".\r\n";
		send(sock, str, str.GetLength(), 0);
		logFile.Write(LOG_ADVANCED, "<%s> Response sent: %s", username, LPCTSTR(str));
	}
	else
	{
		/* Return details for specified message only */
		email = browser.GetEmail(msgNumber);

		if(email != NULL && !email->IsDeleted())
		{
			str.Format("+OK %d %s\r\n", msgNumber, email->GetMsgId());
			send(sock, str, str.GetLength(), 0);
			logFile.Write(LOG_ADVANCED, "<%s> Response sent: %s", username, LPCTSTR(str));
		}
		else
		{
			str.Format("-ERR no such message OR message marked as deleted OR message not downloaded\r\n");
			send(sock, str, str.GetLength(), 0);
			logFile.Write(LOG_ADVANCED, "<%s> Response sent: %s", username, LPCTSTR(str));
		}
	}

	return 0;
}
