/*************************************************************************
*
* YahooPOPs!
*
* YahooPOPs! is an application that provides POP3 access to Yahoo! Mail.
* You do not need to enable POP3 access with Yahoo! to use this program.
* This program works within the license agreement of Yahoo! to provide
* you with POP3 access.
*
* YahooPOPs! emulates a POP3 mail server and should work with most of the
* email clients. Since YahooPOPs! is not a full-fledged POP3 mail server
* you might find it not working with all mail clients. Do keep us informed
* about such mail clients so that we can enhance YahooPOPs!
*
* Official Website: http://yahoopops.sourceforge.net
*
*-------------------------------------------------------------------------
*
* Name    :	SmtpListener.cpp
*
* Purpose : SMTP Server clas
*
* Authors : Anuj Seth, K Shyam, Daniel Bryg
*
* License : GNU Public License (Version 2 or later)
*
*-------------------------------------------------------------------------
*
* THIS SOFTWARE IS PROVIDED ``AS IS'' BY THE AUTHORS AND INDEPENDENT
* CONTRIBUTORS AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
* LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
* PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR 
* CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
* EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
* PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
* PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
* LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
* NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
* SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
*************************************************************************/
#include "stdafx.h"

#ifdef WIN32
#include "ProgressWnd.h"
#else
#include <errno.h>
typedef void* (*THREAD_FUNCTION)(void *);
typedef void* THREAD_PARAMETER;
#endif

#include "DataHolder.h"
#include "resource.h"
#include "LogFile.h"
#include "YahooSessionCache.h"
#include <iostream>
#include <fstream>
#include "ServiceClient.h"
#include "WebBrowser.h"
#include "SmtpListener.h"
#include <mimepp/mimepp.h>
#include "EmailAddressList.h"
#include "StdString.h"
#include "LogFile.h"

extern CDataHolder settings;
extern LogFile     logFile;
extern YahooSessionCache sessionCache;

UINT ServiceSmtpClientThread(void *lpParameter);

class ServiceSmtpClient
{
public:
#ifdef WIN32
	ServiceSmtpClient(SOCKET sock);
#else
	ServiceSmtpClient(SOCKET *sock);
#endif
	~ServiceSmtpClient();

	int Execute(void);

protected:
	int ServiceEHLO(const char *buf);
	int ServiceRSET(const char *buf);
	int ServiceMAIL(const char *buf);
	int ServiceRCPT(const char *buf);
	int ServiceDATA(const char *buf);
	int ServiceNOOP();
	int ServiceQUIT();
	int ServiceMoreDATA(char *data, const int datalen);
	int SendEmail();
	CStdString MakeAddressList(EmailAddressList &list);
    int ServiceAUTHLOGIN(const char *buf);

private:
	CStdString	username;
	CWebBrowser browser;
	SOCKET		sock;
	char		buf[1024];
	int			buflen;
	char		*ptr;
	CStdString	str;
	CStdString	email;
	EmailAddressList rcpt;
	bool		dataMode;
	bool		sessionInit;
#ifdef WIN32
	PROGRESS_DATA *data;
#endif
	/* offset in message where body begins */
	int bodyPos;
	/* current search offset */
	int searchPos;
};

/*******************************************************************
* Function  : SmtpListenThread()
* Parameters: -
* Returns   : 0
* Purpose   : A thread that listens for SMTP client requests
*******************************************************************/
UINT SmtpListenThread(void *lpParameter)
{
	struct sockaddr_in serv;
	struct sockaddr_in cli;
	int    listenfd = 0;
	int    sockfd   = 0;
	int    clilen   = 0;
	int    result   = 0;
	int    retVal   = 0;
	
	if((listenfd = socket(AF_INET, SOCK_STREAM, 0)) < 0)
	{
#ifdef WIN32
		::AfxMessageBox(IDS_SMTP_SOCKET_ERROR, MB_OK | MB_ICONSTOP);
		::PostMessage(::AfxGetMainWnd()->GetSafeHwnd(), WM_QUIT, 0, 0);
        AfxEndThread(0);
#else
		logFile.Write(LOG_ADVANCED,"Unable to open socket %d %s.Reason = %s\n",__LINE__,__FILE__,strerror(errno));
		return(0);
#endif
	}
	
	memset(&serv, 0, sizeof(serv));
	serv.sin_family 	 = AF_INET;
#ifdef WIN32
	serv.sin_addr.s_addr = inet_addr(settings.m_strNetworkIPAddr);
#else
	serv.sin_addr.s_addr = INADDR_ANY;
#endif
	serv.sin_port		 = htons(settings.m_nSmtpPort);
	
	if (bind(listenfd,(struct sockaddr *) &serv, sizeof(serv)) < 0)
	{
#ifdef WIN32
        ::AfxMessageBox(IDS_SMTP_SOCKET_ERROR, MB_OK | MB_ICONSTOP);
		closesocket(listenfd);
        AfxEndThread(0);
#else
		logFile.Write(LOG_ADVANCED,"Unable to bind %d %s.Reason = %s\n",__LINE__,__FILE__,strerror(errno));
		close(listenfd);
#endif
		return(0);
	}
	
	if (listen(listenfd, 25) < 0)
	{
#ifdef WIN32
		::AfxMessageBox(IDS_SMTP_SOCKET_ERROR, MB_OK | MB_ICONSTOP);
		closesocket(listenfd);
		AfxEndThread(0);
#else
		logFile.Write(LOG_ADVANCED,"Unable to listen %d %s.Reason = %s\n",__LINE__,__FILE__,strerror(errno));
		close(listenfd);
#endif
		return(0);
	}
	
	while(true)
	{
		clilen = sizeof(cli);
#ifdef WIN32
		if((sockfd = accept(listenfd,(struct sockaddr *) &cli, &clilen)) < 0)
#else
		if((sockfd = accept(listenfd,(struct sockaddr *) &cli, (socklen_t*)&clilen)) < 0)
#endif
		{
			break;
		}
		else
		{
#ifdef WIN32
			::AfxBeginThread(ServiceSmtpClientThread, (LPVOID)sockfd);
#else
			pthread_t threadId;
			pthread_create(&threadId,NULL,(THREAD_FUNCTION)ServiceSmtpClientThread,(THREAD_PARAMETER)&sockfd);
#endif
		}
	}
			
	// Exiting
#ifdef WIN32
	::PostMessage(::AfxGetMainWnd()->GetSafeHwnd(), WM_QUIT, 0, 0);
    closesocket(listenfd);
	AfxEndThread(0);
#else
	close(listenfd);
#endif
    return 0;
}

/*******************************************************************
* Function  : ServiceSmtpClientThread
* Parameters: lpParameter -> The client socket
* Returns   : Return value from service.Execute()
* Purpose   : To service a SMTP clients request. This is a thread
*             which is spawned by SmtpListenThread()
*******************************************************************/
UINT ServiceSmtpClientThread(void *lpParameter)
{
	int retVal;
#ifndef WIN32
	int sock = 0;
#endif
	try
	{
#ifdef WIN32
		SOCKET sock = (reinterpret_cast<SOCKET>(lpParameter));
		ServiceSmtpClient service(sock);
#else
		sock = *((int*)lpParameter);
		ServiceSmtpClient service(&sock);
#endif
		retVal = service.Execute();
	}
	catch(...)
	{
		/* 
		 * This catch will catch only C++ exceptions.
		 * I don't expect it to catch any C runtime
		 * library exceptions which could be caused
		 * due to the bugs in my code :(
		 */
		retVal = 1;
	}
#ifdef WIN32
    AfxEndThread(retVal);
#else
	close(sock);
#endif
    return 0;
}


#ifdef WIN32
ServiceSmtpClient::ServiceSmtpClient(SOCKET sock)
#else
ServiceSmtpClient::ServiceSmtpClient(SOCKET *sock)
#endif
 {
#ifdef WIN32
 	this->sock = sock;
#else
	this->sock = *sock;
#endif
 	sessionInit = false;
}

ServiceSmtpClient::~ServiceSmtpClient()
{
#ifdef WIN32
	closesocket(sock);
#else
	close(sock);
#endif
}

int ServiceSmtpClient::Execute(void)
{
	CStdString progressStr;
#ifdef WIN32
	data = new PROGRESS_DATA;

	::AfxBeginThread(ProgressThread, data);
	::WaitForSingleObject(data->init, INFINITE);

	logFile.Write(LOG_BASIC, "Starting SMTP session.");

	CString caption;
	caption.Format("Sending emails...");

	if(data->progress != NULL)
	{
		data->progress->SetCaption(caption);
		data->progress->SetRange(0, 100);
		data->progress->SetText("Initializing SMTP transfer...");
	}
#endif

#ifdef WIN32	
	str.Format("220 YahooPOPs! Simple Mail Transfer Service Ready\r\n");
	send(sock, LPCTSTR(str), str.GetLength(), 0);
#else
	char *string = "220 YahooPOPs! Simple Mail Transfer Service Ready\r\n";
	send(sock,string,strlen(string), 0);
#endif

    dataMode = false;

	while(true)
	{
		buflen = recv(sock, buf, 1023, 0);

		if(buflen <= 0)
		{
			/* Socket has been closed */
			break;
		}
#ifdef WIN32
		buf[buflen] = NULL;
#else
		buf[buflen] = '\0';
#endif

#ifdef WIN32
		if(data == NULL || (data->progress != NULL && data->progress->Cancelled()))
		{
			/* User clicked on the Cancel button */
			closesocket(sock);

			logFile.Write(LOG_ADVANCED, "<%s> Cancel event received", username.c_str());

			if(data != NULL)
			{
				data->destroy.SetEvent();
			}

			return 0;
		}
#endif

		if(dataMode == false)
		{
			logFile.Write(LOG_ADVANCED, "<%s> Command received from SMTP client: %s", username.c_str(), buf);
#ifdef WIN32
			if (data->progress != NULL) {
			caption.Format("Request: %s", buf);
			data->progress->SetText(caption);
			}
#endif
		}

		if(dataMode == true)
		{
			ServiceMoreDATA(buf, buflen);
			/* If there's no mora data */
			if(dataMode == false)
			{
#ifdef WIN32
				if(data->progress != NULL)
				{
					data->progress->SetText("Sending message to Yahoo!...");
				}
#endif

                logFile.Write(LOG_ADVANCED, "<%s> Sending message to Yahoo!", username.c_str());
				
                /* Send the email */
				if (0 == SendEmail()) 
                {
					logFile.Write(LOG_BASIC, "<%s> Message sent", username.c_str());
                }
				else 
                {
                    logFile.Write(LOG_BASIC, "<%s> Message not sent", username.c_str());
					logFile.Write(LOG_ADVANCED, "<%s> Removing session from cache", username.c_str());
					browser.DeleteSession();
                }

				email = "";
				/* Clear recipient list */
				rcpt = EmailAddressList(); 
			}
		}
		else if(strnicmp(buf, "EHLO", 4) == 0 || strnicmp(buf, "HELO", 4) == 0)
		{
			ServiceEHLO(buf);
		}
		else if(strnicmp(buf, "RSET", 4) == 0)
		{
			ServiceRSET(buf);
		}
		else if(strnicmp(buf, "MAIL FROM:", 10) == 0)
		{
			ServiceMAIL(buf);
		}
		else if(strnicmp(buf, "RCPT TO:", 8) == 0)
		{
			ServiceRCPT(buf);
		}
		else if(strnicmp(buf, "DATA", 4) == 0)
		{
			ServiceDATA(buf);
#ifdef WIN32
			if(data->progress != NULL)
			{
				data->progress->SetText("Receiving message from client...");
			}
#endif
		}
		else if(strnicmp(buf, "NOOP", 4) == 0)
		{
			ServiceNOOP();
		}
		else if(strnicmp(buf, "QUIT", 4) == 0)
		{
			ServiceQUIT();
		}
		else if(strnicmp(buf, "AUTH LOGIN", 10) == 0)
		{
			ServiceAUTHLOGIN(buf);
		}
		else 
        {
			str.Format("500 Command not recognized\r\n");
			send(sock, str, str.GetLength(), 0);
			logFile.Write(LOG_ADVANCED, "<%s> SMTP response sent: %s", username.c_str(), str.Buffer());
		}
	}
#ifdef WIN32
	data->destroy.SetEvent();
#endif
	return 0;
}

int ServiceSmtpClient::ServiceEHLO(const char *buf)
{
	str.Format("250-YahooPOPs! greets you.\r\n250 AUTH LOGIN\r\n");
	send(sock, str, str.GetLength(), 0);
	logFile.Write(LOG_ADVANCED, "<%s> SMTP response sent: %s", username.c_str(), str.Buffer());
	return 0;
}

int ServiceSmtpClient::ServiceRSET(const char *buf)
{
	email = "";
	/* Clear recipient list */
	rcpt = EmailAddressList(); 

	str.Format("250 OK\r\n");
	send(sock, str, str.GetLength(), 0);
	logFile.Write(LOG_ADVANCED, "<%s> SMTP response sent: %s", username.c_str(), str.Buffer());
	return 0;
}

int ServiceSmtpClient::ServiceMAIL(const char *buf)
{
	char yahooEmail[128];
	
	email = "";
	
	sscanf(buf, "%*4s %*4s: <%s>\r\n", yahooEmail);
		
	if(strchr(yahooEmail, '@') == NULL)
	{
		/*
		 * Received an unsupported format of email address
		 */
		str.Format("553 Bad mailbox name.\r\n");
		send(sock, str, str.GetLength(), 0);
		logFile.Write(LOG_ADVANCED, "<%s> SMTP response sent: %s", username.c_str(), str.Buffer());
	         return 1;
	}

	*strchr(yahooEmail, '@') = '\0';
	this->username=yahooEmail;

	if(!sessionInit)
	{
		//EDIT: Shane.H : Pass a NULL password to skip password check
		YAHOO_SESSION *ySession = sessionCache.GetSession(yahooEmail);
		
		if(ySession == NULL)
		{
			/* No session for this user exists */
			str.Format("530 POP authenticate before sending emails.\r\n");
			send(sock, str, str.GetLength(), 0);
			return 1;
		}

		browser.DeleteSession();
		browser.SetSession(ySession);
		sessionInit = true;
	}
	
	str.Format("250 OK\r\n");
	send(sock, str, str.GetLength(), 0);
	logFile.Write(LOG_ADVANCED, "<%s> SMTP response sent: %s", username.c_str(), str.Buffer());
	return 0;
}

int ServiceSmtpClient::ServiceRCPT(const char *buf)
{
	/* Extract recipient from "RCPT TO: <addr>" 
	   and add to the list */
	char newaddr[256];

    if ((ptr = strchr(buf, '<')) !=NULL) 
    {
		int i = strcspn(++ptr,">");
		strncpy(newaddr, ptr, i);
		/* end with null */
		newaddr[i] = 0;

        rcpt.Add(EmailAddress(newaddr,"",""));
	}

	str.Format("250 OK\r\n");
	send(sock, str, str.GetLength(), 0);
	logFile.Write(LOG_ADVANCED, "<%s> SMTP response sent: %s", username.c_str(), str.Buffer());
	return 0;
}

int ServiceSmtpClient::ServiceDATA(const char *buf)
{
	str.Format("354 OK\r\n");
	send(sock, str, str.GetLength(), 0);
	logFile.Write(LOG_ADVANCED, "<%s> SMTP response sent: %s", username.c_str(), str.Buffer());

	dataMode = true;
	/* offset in message where body begins */
	bodyPos=-1;
	/* current search offset */
	searchPos=0;

	return 0;
}

int ServiceSmtpClient::ServiceAUTHLOGIN(const char *buf)
{
	int res;
	char resp[1024];
	char* ptr;

	/*
		When there is no parameter to AUTH LOGIN command
		send a request for username.
	*/
	if (buf[10]=='\r') 
    {
		str.Format("334 VXNlcm5hbWU6\r\n"); //base64 - "Username:"
		send(sock, str, str.GetLength(), 0);
		logFile.Write(LOG_ADVANCED, "<%s> SMTP response sent: %s", username.c_str(), str.Buffer());
		
		if ((res=recv(sock, resp, 1023, 0))<=0) 
        {
            return -1;
        }
    	//Connection closed
		
		if (resp[0]=='*') 
        {
			str.Format("501 Auth aborted\r\n");
			send(sock, str, str.GetLength(), 0);
			logFile.Write(LOG_ADVANCED, "<%s> SMTP response sent: %s", username.c_str(), str.Buffer());
			return -1;
		}
		
		resp[res]=0;
		ptr=strstr(resp,"\r\n");
		
        if (ptr==NULL) 
        {
			str.Format("501 Bad response\r\n");
			send(sock, str, str.GetLength(), 0);
			logFile.Write(LOG_ADVANCED, "<%s> SMTP response sent: %s", username.c_str(), str.Buffer());
			return -1;
		}
		
        *ptr=0;	//remove the newline characters
	}
	/*
		When there is a parameter to AUTH LOGIN command
		(the character that follows is a space), extract username.
	*/
	else if (buf[10]==' ') 
    {
		sscanf(buf, "%*4s %*5s %s\r\n", resp);
	} 
	else 
    {
		/* AUTH LOGIN is followed by a nonspace character*/
		str.Format("500 Syntax error\r\n");
		send(sock, str, str.GetLength(), 0);
		logFile.Write(LOG_ADVANCED, "<%s> SMTP response sent: %s", username.c_str(), str.Buffer());
		return -1;
	}

	//Decode username
	DwString username(resp);
	
    if (DwDecodeBase64(username, username)!=0) 
    {
		//b64 decoding error
		str.Format("501 Bad response\r\n");
		send(sock, str, str.GetLength(), 0);
		logFile.Write(LOG_ADVANCED, "<%s> SMTP response sent: %s", username.c_str(), str.Buffer());
		return -1;
	}

	//Send request for password
	str.Format("334 UGFzc3dvcmQ6\r\n");	//base64 - "Password:"
	send(sock, str, str.GetLength(), 0);
	logFile.Write(LOG_ADVANCED, "<%s> SMTP response sent: %s", username.c_str(), str.Buffer());

	if ((res=recv(sock, resp, 1023, 0))<=0)  //Connection closed
    {
        return -1;
    }

	if (resp[0]=='*') 
    {
		str.Format("501 Auth aborted\r\n");
		send(sock, str, str.GetLength(), 0);
		logFile.Write(LOG_ADVANCED, "<%s> SMTP response sent: %s", username.c_str(), str.Buffer());
		return -1;
	}

	resp[res] = 0;
	ptr = strstr(resp,"\r\n");
	
    if (ptr==NULL) 
    {
		str.Format("501 Bad response\r\n");
		send(sock, str, str.GetLength(), 0);
		logFile.Write(LOG_ADVANCED, "<%s> SMTP response sent: %s", username.c_str(), str.Buffer());
		return -1;
	}
	*ptr=0;

	//Decode password
	DwString password(resp);
	
    if (DwDecodeBase64(password, password)!=0) 
    {
		//b64 decoding error
		str.Format("501 Bad response\r\n");
		send(sock, str, str.GetLength(), 0);
		logFile.Write(LOG_ADVANCED, "<%s> SMTP response sent: %s", username.c_str(), str.Buffer());
		return -1;
	}

	if (browser.ConnectToYahoo((char*)username.c_str(),(char*)password.c_str())!=WEB_LOGIN_OK) 
    {
		str.Format("535 Login failed\r\n");
		send(sock, str, str.GetLength(), 0);
		logFile.Write(LOG_ADVANCED, "<%s> SMTP response sent: %s", username.c_str(), str.Buffer());
		return -1;			//Login failed
	}

	str.Format("235 Login successful\r\n");
	send(sock, str, str.GetLength(), 0);
	logFile.Write(LOG_ADVANCED, "<%s> SMTP response sent: %s", username.c_str(), str.Buffer());

	this->username = username.c_str();

	sessionInit = true;

	//Session is already saved in connectToYahoo (if it was successful)
	//browser.SaveSession();

	return 0;
}

int ServiceSmtpClient::ServiceNOOP()
{
	str.Format("250 OK\r\n");
	send(sock, str, str.GetLength(), 0);
	logFile.Write(LOG_ADVANCED, "<%s> SMTP response sent: %s", username.c_str(), str.Buffer());

	return 0;
}

int ServiceSmtpClient::ServiceQUIT()
{
	str.Format("221 YahooPOPs! SMTP service closing transmission channel\r\n");
	send(sock, str, str.GetLength(), 0);
	logFile.Write(LOG_ADVANCED, "<%s> SMTP response sent: %s", username.c_str(), str.Buffer());

	return 0;
}

int ServiceSmtpClient::ServiceMoreDATA(char* data, const int datalen)
{
	/* In case of binary data (client's bug), ignore it
	   and change all zeros to -1's */
	for (int i=0;i<datalen;i++) if (!data[i]) data[i]=(char)-1;

	email += data;

	/* if still downloading headers, find header terminator (\r\n\r\n) */
	if (bodyPos<0) 
	{
		if ((bodyPos=email.Find("\r\n\r\n", searchPos))>=0) 
		{
			/* header terminator found - set body offset */
			bodyPos+=4;
			/* set search offset at the beginning of body */
			searchPos=bodyPos;
		} 
		else 
		{
			/* header terminator not found - set search offset 
			   3 positions before current end of buffer */
			searchPos=email.GetLength()-3;
			if (searchPos<0) searchPos=0;
			return 0;
		}
	}

	/* Headers are complete */
	
	/* Position of body terminator */
	int dotPos=-1;

	/* if search offset is at the beginning of body,
	   check for a short body terminator (.\r\n) there */
	if (bodyPos==searchPos)
		if (!memcmp(email.c_str()+searchPos, ".\r\n", 3))
			dotPos=searchPos;

	/* short terminator hasn't been found, search for 
	   a long terminator (\r\n.\r\n) in body.
	   If short or long body terminator found, 
	   cut it off and end data mode
	*/
	if (dotPos>=0 || (dotPos = email.Find("\r\n.\r\n", searchPos)) >= 0)
	{
		email.Delete(dotPos, email.GetLength() - dotPos);
		dataMode = false;
	}
	else
	{
		/* still looking for body terminator - 
		   search offset 4 bytes from the current buffer end */
		searchPos=email.GetLength()-4;
		if (searchPos<bodyPos) searchPos=bodyPos;
	}

	return 0;
}

CStdString ServiceSmtpClient::MakeAddressList(EmailAddressList &list)
{
	CStdString format;
	CStdString emails;

	int listcount = list.Count();

	/* Loop through addresses in the list */
	for(int i = 0; i < listcount; i++)
	{
		EmailAddress e = list.Get(i);

		/* Add the address if present in SMTP recipients */
		int rcptcount = rcpt.Count();

		for (int j=0; j < rcptcount; j++)
		{
			if (rcpt.Get(j).InetName()==e.InetName())
			{
				/* Ugly - the EmailAddressList should just allow to remove an item */
				((EmailAddress &)rcpt.Get(j)).Set("","","");

				/* Add address to the string */
				format.Format("%s <%s>", e.PersonalName().c_str(), e.InetName().c_str());
				if(!emails.IsEmpty())
				{
					emails += ", ";
				}
				emails += format;
				break;
			}
		}

	}

	return emails;
}

int ServiceSmtpClient::SendEmail()
{
	EmailMessage msg;

	msg.ParseFrom(DwString(email));

	CStdString subject(msg.Subject().Text().c_str());

	CStdString to  = MakeAddressList(msg.ToRecipients());
	CStdString cc  = MakeAddressList(msg.CcRecipients());
	CStdString bcc = MakeAddressList(msg.BccRecipients());

	/* Add BCC recipients not present in To, Cc and Bcc headers
	 * but requested with RCPT command
	 */
	for (int n = 0; n < rcpt.Count(); n++) 
    {
		/* Due to implementation of EmailAddressList we could not remove
		   used addresses from the recipient list - we have set them to "" instead) */
		if (!rcpt.Get(n).InetName().empty()) {
		CStdString angleaddr;
		angleaddr.Format("<%s>", rcpt.Get(n).InetName().c_str());
	
			if (!bcc.IsEmpty()) 
                bcc += ",";	//don't add comma for the first address
			bcc += angleaddr;
		}
	}

	logFile.Write(LOG_ADVANCED, "<%s> The message will be sent to:\n\
		TO:%s,CC:%s,BCC:%s", username.c_str(), to.c_str(), cc.c_str(), bcc.c_str());

	/*
	 * Extract html portion of the message if it exists
	 */
	bool htmlbody=false;
	DwString dwBody;
	DwMessage message=DwMessage(DwString(email));
	message.Parse();
	DwEntity* dwEnt=&message;;

	while(true) 
    {
		if (dwEnt->Headers().HasContentType() 
			&& dwEnt->Headers().ContentType().Type()==DwMime::kTypeText
			&& dwEnt->Headers().ContentType().Subtype()==DwMime::kSubtypeHtml) 
        {
			/* html portion found */
 			htmlbody=true;
			break; /* while */
		} 
        else if (dwEnt->Headers().HasContentType() 
			&& dwEnt->Headers().ContentType().Type()==DwMime::kTypeMultipart) 
        {
			switch (dwEnt->Headers().ContentType().Subtype()) 
            {
			case DwMime::kSubtypeAlternative:
				/*
				  Go into the second subpart.
				 */
				dwEnt=&dwEnt->Body().BodyPartAt(1);
				break; /* switch */
			default:
				/*
				  Go into the first subpart.
				 */
				dwEnt=&dwEnt->Body().BodyPartAt(0);
			}
		}
		else 
        {
            break; /* while */
        }
	}

	/* If text/html part has't been found at the beginning of the message
	   extract plain text
	 */
	if (!htmlbody) 
    {
		dwBody=msg.MemoText().Text().c_str();
		/* Make sure each line ends with CRLF */
		DwToCrLfEol(dwBody, dwBody);
	} 
    else 
    {
	/* Extract text/html from the found part */
		dwBody=dwEnt->Body().AsString();

		/* Decode "quoted-printable" */
		if (dwEnt->Headers().HasContentTransferEncoding() 
			&& (0 == DwStrcasecmp(dwEnt->Headers().ContentTransferEncoding().AsString(), 
			"quoted-printable"))) 
        {
			DwDecodeQuotedPrintable(dwBody, dwBody);
        }
	}

	EmailAttachmentList &attachment = msg.Attachments();

	/* try send the message and create a response to the client */
	CStdString DwBody = dwBody.c_str();
	int n = browser.SendEmail(to, cc, bcc, subject,DwBody, htmlbody, attachment);
	switch (n)
    {
	case 1:
		str.Format("554 Mailbox unavailable.\r\n");
		break;
	case 2:
		str.Format("554 Cannot send message.\r\n");
		break;
	case 3:
		str.Format("554 Cannot send attachments.\r\n");
		break;
	case 0:
		str.Format("250 OK\r\n");
		break;
	default:
		/* should not happen */
		str.Format("554 Unknown error while sending");
	}

	/* send response to the client */
	send(sock, str, str.GetLength(), 0);
	logFile.Write(LOG_ADVANCED, "<%s> SMTP response sent: %s", username.c_str(), str.Buffer());

	/* return failure or success, depending on result of browser.SendEmail */
	return n?1:0;
}
