/*************************************************************************
*
* YahooPOPs!
*
* YahooPOPs! is an application that provides POP3 access to Yahoo! Mail.
* You do not need to enable POP3 access with Yahoo! to use this program.
* This program works within the license agreement of Yahoo! to provide
* you with POP3 access.
*
* YahooPOPs! emulates a POP3 mail server and should work with most of the
* email clients. Since YahooPOPs! is not a full-fledged POP3 mail server
* you might find it not working with all mail clients. Do keep us informed
* about such mail clients so that we can enhance YahooPOPs!
*
* Official Website: http://yahoopops.sourceforge.net
*
*-------------------------------------------------------------------------
*
* Name    :	YahooPOPs.cpp
*
* Purpose : The main class for Windows
*
* Author  : Anuj Seth
* 			K.Shyam
*
* License : GNU Public License (Version 2 or later)
*
*-------------------------------------------------------------------------
*
* THIS SOFTWARE IS PROVIDED ``AS IS'' BY THE AUTHORS AND INDEPENDENT
* CONTRIBUTORS AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
* LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
* PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR 
* CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
* EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
* PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
* PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
* LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
* NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
* SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
*************************************************************************/
#include "stdafx.h"

#ifdef WIN32
#include <openssl/ssl.h>
#include <curl/curl.h>
#include <curl/types.h>
#include <regexp.h>
#include "YahooPOPs.h"
#include "MainFrm.h"
#include "DataHolder.h"
#include "ServiceClient.h"
#include "YahooStuff.h"
#include "DisclaimerDlg.h"
#include "LogFile.h"
#include "YahooSessionCache.h"

#define _WINSOCK2API_	// To ensure that MIME++ does not crib!
#include <mimepp/mimepp.h>
#include "SmtpListener.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

CSystemTrayApp		theApp;
CDataHolder			settings;
LogFile				logFile("yahoopops.log");
YahooSessionCache	sessionCache;
CMainFrame			*gp_MainFrame = NULL;

static void regexp_error(char* s)
{
	/* Do nothing */
}

BEGIN_MESSAGE_MAP(CSystemTrayApp, CWinApp)
	//{{AFX_MSG_MAP(CSystemTrayApp)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

CSystemTrayApp::CSystemTrayApp() : mutInstance (NULL)
{
}

/*******************************************************************
* Function  : InitInstance()
* Parameters: -
* Returns   : BOOL
* Purpose   : If you know MFC, you know what this function does.
*******************************************************************/
BOOL CSystemTrayApp::InitInstance()
{

	if (mutInstance) return FALSE; // Instance already initialized 

	mutInstance = CreateMutex (NULL, FALSE, "YAHOOPOPS!_instance_mutex");
	if (mutInstance == NULL) {
		AfxMessageBox ("Can't create mutex!");   
		return FALSE;
	}

	// EDIT Shane .H: If the /QUIT argument was specified, close exisiting instances
	if ((__argc > 1) && !stricmp(__argv[1], "/QUIT"))
	{
		CloseHandle (mutInstance);
		mutInstance=NULL;
		::PostMessage( HWND_BROADCAST, insMessage, 1, 0L );
		//And don't start
		return FALSE;
	}
	
	if (GetLastError() == ERROR_ALREADY_EXISTS) {

		CloseHandle (mutInstance);
		mutInstance=NULL;

		//EDIT :Shane .H  Silent start mode - don't show configure dialog box
		if (!((__argc > 1) && !stricmp(__argv[1], "/SILENT")))
		::PostMessage( HWND_BROADCAST, insMessage, 0, 0L );

		return FALSE;  //don't start 2nd instance
	}
	
	WSADATA wsaData;
	if (::WSAStartup(0x202,&wsaData)) {
		AfxMessageBox ("Cannot initialize WinSock library. YahooPOPs will fail to function.");   
		return FALSE;
	}

#ifdef _AFXDLL
	Enable3dControls();			// Call this when using MFC in a shared DLL
#else
	Enable3dControlsStatic();	// Call this when linking to MFC statically
#endif

	SetRegistryKey(_T("YahooPOPs!"));
	settings.Load();

	CMainFrame* pMainFrame = new CMainFrame;
	gp_MainFrame = pMainFrame;
	
	if (!pMainFrame->LoadFrame(IDR_MAINFRAME))
	{
		return FALSE;
	}

	m_pMainWnd = pMainFrame;

	YahooSetup();

	/* Configure regular expression library */
	set_regerror_func(regexp_error);

	BOOL m_bDisclaimerRead = GetProfileInt("Settings", "DisclaimerRead0.5.0", FALSE);

	if(!m_bDisclaimerRead)
	{
		CDisclaimerDlg dlg;
		
		if(dlg.DoModal() != IDOK)
		{
			::PostMessage(::AfxGetMainWnd()->GetSafeHwnd(), WM_QUIT, 0, 0);
		}
		else
		{
			WriteProfileInt("Settings", "DisclaimerRead0.5.0", TRUE);
		}
	}

	/* Initialize the OpenSSL library */
	CRYPTO_malloc_init();
	SSLeay_add_all_algorithms();
	OpenSSL_add_ssl_algorithms();
	SSL_load_error_strings();

	/* Initialize the MIME library */
	DwInitialize();

	/* Initialize curl */
	curl_global_init(CURL_GLOBAL_ALL);

  	CString thisVersion;
    thisVersion.LoadString(IDS_VERSION);
    logFile.Write(LOG_BASIC, "YahooPOPs! v%s started successfully", LPCTSTR(thisVersion));

    settings.DumpToLog();

	::AfxBeginThread(ListenThread, NULL);

    if(settings.m_bEnableSmtp)
    {
	    ::AfxBeginThread(SmtpListenThread, NULL);
    }

	return TRUE;
}

/*******************************************************************
* Function  : ExitInstance()
* Parameters: -
* Returns   : The application's exit code
* Purpose   : If you know MFC, you know what this function does.
*******************************************************************/
int CSystemTrayApp::ExitInstance()
{
	logFile.Write(LOG_BASIC, "YahooPOPs! is exiting.");
	if ( mutInstance ) 
    {
		CloseHandle (mutInstance);
		mutInstance=NULL;
	}
	
	return CWinApp::ExitInstance();
}


/******************************************************************
* CAboutDlg code follows....
*******************************************************************/
CAboutDlg::CAboutDlg() : CDialog(CAboutDlg::IDD)
{
	//{{AFX_DATA_INIT(CAboutDlg)
	//}}AFX_DATA_INIT
}

void CAboutDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CAboutDlg)
	DDX_Control(pDX, IDC_TEAM_LIST, m_ctlTeamList);
	DDX_Control(pDX, IDC_YAHOOPOP_URL, m_urlYahooPop);
	DDX_Control(pDX, IDC_ACK_LIST, m_ctlAckList);
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CAboutDlg, CDialog)
	//{{AFX_MSG_MAP(CAboutDlg)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/*******************************************************************
* Function  : OnInitDialog() 
* Parameters: -
* Returns   : BOOL
* Purpose   : If you know MFC, you know what this function does.
*******************************************************************/
BOOL CAboutDlg::OnInitDialog() 
{
	CDialog::OnInitDialog();
	
    m_ctlTeamList.AddString("The YahooPOPs! Team:");
    m_ctlTeamList.AddString("1. Anuj Seth - Project Lead / Website Administrator");
    m_ctlTeamList.AddString("2. K Shyam  - Developer");
    m_ctlTeamList.AddString("3. Daniel Bryg - Developer / Tech Support");
    m_ctlTeamList.AddString("4. Nick Haas  - Developer / Tech Support");
    m_ctlTeamList.AddString("5. Shane Hird - Developer / Tech Support");

	m_ctlAckList.AddString("YahooPOPs! was inspired by Ravi Ramkissoon's FetchYahoo; a Perl");
	m_ctlAckList.AddString("program that fetches email from Yahoo for mbox-compliant mail clients like");
	m_ctlAckList.AddString("Pine and Netscape.");
	m_ctlAckList.AddString("");
	m_ctlAckList.AddString("YahooPOPs! uses the following third-party libraries:");
	m_ctlAckList.AddString("1. OpenSSL 0.9.7c for establishing secure (SSL) connections.");
	m_ctlAckList.AddString("   \"This product includes cryptographic software written by Eric Young\"");
	m_ctlAckList.AddString("");
	m_ctlAckList.AddString("2. Re-Lib 1.0 for regular expressions.");
	m_ctlAckList.AddString("   \"Copyright (c) Anatoly Kardash <akardash@hotmail.com> and");
	m_ctlAckList.AddString("   Copyright (c) 1986 by University of Toronto.");
	m_ctlAckList.AddString("   Written by Henry Spencer.  Not derived from licensed software.\"");
	m_ctlAckList.AddString("");
	m_ctlAckList.AddString("3. CTrayIcon By Chris Maunder <Chris.Maunder@cbr.clw.csiro.au>");
	m_ctlAckList.AddString("");
	m_ctlAckList.AddString("4. CStringEx::Scanf Extension (http://www.codeguru.com)");
	m_ctlAckList.AddString("    Copyright (c) Rainer Bauer, 2001");	
	m_ctlAckList.AddString("");
	m_ctlAckList.AddString("5. CStdString");
	m_ctlAckList.AddString("    Copyright (c) 1999 Joseph M. O'Leary");
	m_ctlAckList.AddString("");
	m_ctlAckList.AddString("6. Ghost Installer Studio");
	m_ctlAckList.AddString("    Copyright (c) 2001-2002, gInstall Corp.");
	m_ctlAckList.AddString("");
	m_ctlAckList.AddString("7. CProgressWnd");
	m_ctlAckList.AddString("    Copyright (c) 1998 Chris Maunder (chrismaunder@codeguru.com)");
	m_ctlAckList.AddString("");
	m_ctlAckList.AddString("8. HyperLink static control");
	m_ctlAckList.AddString("    Copyright (c) 1997, 1998 Giancarlo Iovino (giancarlo@saria.com)");
	m_ctlAckList.AddString("");
	m_ctlAckList.AddString("9. Options Tree Dialog (http://www.codeguru.com/dialog/optstree.shtml)");
	m_ctlAckList.AddString("    Copyright (c) Petr Stejskal (petr.stejskal@vslib.cz)");
	m_ctlAckList.AddString("");
	m_ctlAckList.AddString("10. MIME++ v1.3.3");
	m_ctlAckList.AddString("     Copyright (c) 1996-2001 Hunny Software, Inc. All rights reserved.");
	m_ctlAckList.AddString("");
	m_ctlAckList.AddString("11. Win32 Code Tips and Handy Functions");
	m_ctlAckList.AddString("     Copyright (c) Natarajan T");
	m_ctlAckList.AddString("");
	m_ctlAckList.AddString("12. cURL (http://curl.haxx.se/)");
    m_ctlAckList.AddString("     Copyright (C) 1998 - 2002, Daniel Stenberg, <daniel@haxx.se>, et al.");

	UpdateData(FALSE);

	m_urlYahooPop.ModifyLinkStyle(0, CHyperLink::StyleUseHover);
		
	return TRUE;
}
#else
#include <iostream.h>
#include <curl/curl.h>
#include <curl/types.h>
#include "YAHOOPOPS.h"
#include "DataHolder.h"
#include "ServiceClient.h"
#include "YahooStuff.h"
#include "LogFile.h"
#include "regexp.h"
#include <pthread.h>
#include "YahooSessionCache.h"
#include <mimepp/mimepp.h>
#include "SmtpListener.h"
#include <openssl/ssl.h>
#include <openssl/md5.h>
#include <openssl/crypto.h>
#include <openssl/evp.h>
#include <unistd.h>

YahooSessionCache	sessionCache;
CDataHolder		settings;
LogFile			logFile("/yahoopops.log");
//LocalizationInfo   locInfo;



typedef void* (*THREAD_FUNCTION)(void *);

int *listenThreadFd;

static void regexp_error(char* s)
{
	/* Do nothing */
}

void listenThreadPort(int *listenfd)
{
       listenThreadFd = listenfd;
}

void handlerFunc(int signal)
{
       logFile.Write(LOG_BASIC, "YahooPops! Terminated due to user abort CTRL + C received.\n");
       close(*listenThreadFd);	
	   sleep(1);	
       exit(0);
}

/* ok the begin all and end all of all *ix systems */
int main()
{
	int error = 0;
	int threadParam = 0;
	pthread_t threadId;

	/* set up the ctrl + c handler */
	signal(SIGINT,handlerFunc);
	if ( settings.Load() == 0 )
	{
	     return(0);
	}
	YahooSetup();

	/* Configure regular expression library */
	set_regerror_func(regexp_error);

	CRYPTO_malloc_init();
	SSLeay_add_all_algorithms();
	OpenSSL_add_ssl_algorithms();
	SSL_load_error_strings();

	/* Initialize the MIME library */
	DwInitialize();

	/* initialize curl */
	curl_global_init(CURL_GLOBAL_ALL);

	settings.DumpToLog();

	/* initialize locale info */
	//as of version 0.6 no more localization shit.goodbye xml.
	//locInfo.Initialize();
	//locInfo.ReadFile();

	
	/*if ( ListenThread((void*)0) != 0 )
	{
		logFile.Write(LOG_BASIC, "Unable to start YahooPOPs!.\n");
	}*/
    if(settings.m_bEnableSmtp)
    {
		threadParam = 1;
		if ( ( error = pthread_create(&threadId,NULL,(THREAD_FUNCTION)ListenThread,&threadParam) ) != 0 )
		{
			fprintf(stderr,"Major error occurred.Unable to start the thread.Error reason = %s\n",strerror(error));
			return(1);
		}

	    if ( SmtpListenThread((void*)0) != 0 )
	    {
		   logFile.Write(LOG_BASIC, "Unable to start YahooPOPs! SMTP service.\n");
	    }
    }
	else
	{
		if ( ListenThread((void*)0) != 0 )
		{
			logFile.Write(LOG_BASIC, "Unable to start YahooPOPs!.\n");
		}
	}
	return(0);
}

#endif
