/*************************************************************************
*
* YahooPOPs!
*
* YahooPOPs! is an application that provides POP3 access to Yahoo! Mail.
* You do not need to enable POP3 access with Yahoo! to use this program.
* This program works within the license agreement of Yahoo! to provide
* you with POP3 access.
*
* YahooPOPs! emulates a POP3 mail server and should work with most of the
* email clients. Since YahooPOPs! is not a full-fledged POP3 mail server
* you might find it not working with all mail clients. Do keep us informed
* about such mail clients so that we can enhance YahooPOPs!
*
* Official Website: http://yahoopops.sourceforge.net
*
*-------------------------------------------------------------------------
*
* Name    :	mainfrm.cpp
*
* Purpose : MFCs MainFrame implementation for YahooPOPs!
*
* Authors : Anuj Seth
* 			K.Shyam
*
* License : GNU Public License (Version 2 or later)
*
*-------------------------------------------------------------------------
*
* THIS SOFTWARE IS PROVIDED ``AS IS'' BY THE AUTHORS AND INDEPENDENT
* CONTRIBUTORS AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
* LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
* PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR 
* CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
* EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
* PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
* PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
* LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
* NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
* SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
*************************************************************************/
#include "stdafx.h"

#ifdef WIN32
#include "YahooPOPs.h"
#include "MainFrm.h"
#include "YahooPops.h"
#include "OptionsTreeDlg.h"
#include "DataHolder.h"
#include "HttpClient.h"
#include "relib_enhance.h"
#include "ProgressWnd.h"
#include "UpdateYahooPOPsDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#define	WM_ICON_NOTIFY (WM_USER + 10)

extern CSystemTrayApp theApp;
extern CDataHolder settings;

BOOL bConfigureVisible = FALSE;

IMPLEMENT_DYNCREATE(CMainFrame, CFrameWnd)

BEGIN_MESSAGE_MAP(CMainFrame, CFrameWnd)
	//{{AFX_MSG_MAP(CMainFrame)
	ON_WM_CREATE()
	ON_COMMAND(ID_POPUP_CONFIGURE, OnPopupConfigure)
	ON_REGISTERED_MESSAGE(insMessage, OnInstanceMessage)
	ON_COMMAND(ID_APP_ABOUT, OnPopupAbout)
	ON_COMMAND(ID_POPUP_HELP, OnPopupHelp)
	ON_COMMAND(ID_POPUP_CHECK_FOR_NEW_VERSION, OnCheckForNewVersion)
	//}}AFX_MSG_MAP
	ON_MESSAGE(WM_ICON_NOTIFY, OnTrayNotification)
END_MESSAGE_MAP()

CMainFrame::CMainFrame()
{
}

CMainFrame::~CMainFrame()
{
}

int CMainFrame::OnCreate(LPCREATESTRUCT lpCreateStruct) 
{
	if (CFrameWnd::OnCreate(lpCreateStruct) == -1)
	{
		return -1;
	}
	
	if (!settings.m_bHideTrayIcon) {
		// Create the tray icon
		if (!m_TrayIcon.Create(this, WM_ICON_NOTIFY, _T("YahooPOPs!"), NULL, IDR_POPUP_MENU))
		{
			return -1;
		}

		m_TrayIcon.SetIcon(IDR_MAINFRAME);
	}
	
	return 0;
}

LRESULT CMainFrame::OnTrayNotification(WPARAM wParam, LPARAM lParam)
{
	return m_TrayIcon.OnTrayNotification(wParam, lParam);
}

/*******************************************************************
* Function  : OnPopupConfigure()
* Parameters: -
* Returns   : -
* Purpose   : To display the settings dialog
*******************************************************************/
void CMainFrame::OnPopupConfigure()
{
	SetForegroundWindow();
	SetFocus();
	if(bConfigureVisible == FALSE)
	{
		bConfigureVisible = TRUE;
		COptionsTreeDlg dlg;
		dlg.DoModal();
	}
}

LRESULT CMainFrame::OnInstanceMessage(WPARAM w, LPARAM l)
{
	if (w == 1)
	{
		PostMessage(WM_CLOSE, 0, 0);
	}
	else OnPopupConfigure();

	return 0;
}

/*******************************************************************
* Function  : OnPopupHelp()
* Parameters: -
* Returns   : -
* Purpose   : To launch the YahooPOPs! Help system
*******************************************************************/
void CMainFrame::OnPopupHelp()
{
	if(ShellExecute(NULL, "open", "YahooPOPs.chm", 0, 0, SW_MAXIMIZE) == HINSTANCE(0x02))
	{
		AfxMessageBox("Unable to find the help file. Help files are distributed with the full version of YahooPOPs! only. The compact version of YahooPOPs! does not ship with the help file.", MB_OK | MB_ICONSTOP);
	}
}

/*******************************************************************
* Function  : OnPopupAbout()
* Parameters: -
* Returns   : -
* Purpose   : To display the About box
*******************************************************************/
void CMainFrame::OnPopupAbout() 
{
	CAboutDlg dlg;
	dlg.DoModal();
}

/*******************************************************************
* Function  : OnCheckForNewVersion()
* Parameters: -
* Returns   : -
* Purpose   : 
*******************************************************************/
void CMainFrame::OnCheckForNewVersion() 
{
	HttpSession session;
	HttpClient c(&session);
	HttpClientResponse resp;

	if(settings.m_bUseProxy)
	{
		c.UseHttpProxy(TRUE);
		c.SetHttpProxy(LPCTSTR(settings.m_strHttpProxyHost), settings.m_nHttpProxyPort);
	}

	if(c.DoGet("http://yahoopops.sourceforge.net/version_windows.txt", resp) != 0)
	{
		AfxMessageBox("Unable to retrieve version information from YahooPOPs! server", MB_OK | MB_ICONEXCLAMATION);
		return;
	}

	char version[80];
	char relDate[20];
	char downloadUrl[1024];

	char *ptr = NULL;
	int retVal = SearchRegExp(resp.GetBodyPtr(), "Version=",
						  &ptr, "Version=%s\n", version);

	ptr = NULL;
	retVal = SearchRegExp(resp.GetBodyPtr(), "ReleaseDate=",
						  &ptr, "ReleaseDate=%s\n", relDate);

	ptr = NULL;
	retVal = SearchRegExp(resp.GetBodyPtr(), "DownloadURL=",
						  &ptr, "DownloadURL=%s", downloadUrl);

	CString thisVersion;
	thisVersion.LoadString(IDS_VERSION);

	if(thisVersion == version)
	{
		AfxMessageBox("You have the latest version of YahooPOPs!", MB_OK | MB_ICONINFORMATION);
	}
	else
	{
		CUpdateYahooPOPsDlg dlg;

		dlg.m_strDownloadUrl = downloadUrl;
		dlg.m_strReleaseDate = relDate;
		dlg.m_strVersion	 = version;
		
		dlg.DoModal();
	}
}

#endif // #ifdef WIN32

